/****************************************************************************
** Meta object code from reading C++ file 'guiconfig.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/core/config/guiconfig.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'guiconfig.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9GuiConfigE_t {};
} // unnamed namespace

template <> constexpr inline auto GuiConfig::qt_create_metaobjectdata<qt_meta_tag_ZN9GuiConfigE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "GuiConfig",
        "fileListSortColumnChanged",
        "",
        "fileListSortColumn",
        "fileListSortOrderChanged",
        "Qt::SortOrder",
        "fileListSortOrder",
        "fileListVisibleColumnsChanged",
        "QList<int>",
        "fileListVisibleColumns",
        "fileListCustomColumnWidthsEnabledChanged",
        "fileListCustomColumnWidthsEnabled",
        "fileListColumnWidthsChanged",
        "fileListColumnWidths",
        "dirListSortColumnChanged",
        "dirListSortColumn",
        "dirListSortOrderChanged",
        "dirListSortOrder",
        "dirListVisibleColumnsChanged",
        "dirListVisibleColumns",
        "dirListCustomColumnWidthsEnabledChanged",
        "dirListCustomColumnWidthsEnabled",
        "dirListColumnWidthsChanged",
        "dirListColumnWidths",
        "splitterSizesChanged",
        "splitterSizes",
        "vSplitterSizesChanged",
        "vSplitterSizes",
        "autoHideTagsChanged",
        "autoHideTags",
        "hideFileChanged",
        "hideFile",
        "hideTagChanged",
        "hidePictureChanged",
        "hidePicture",
        "playOnDoubleClickChanged",
        "playOnDoubleClick",
        "selectFileOnPlayEnabledChanged",
        "selectFileOnPlayEnabled",
        "playToolBarVisibleChanged",
        "playToolBarVisible",
        "preferredAudioOutputChanged",
        "preferredAudioOutput",
        "playToolBarAreaChanged",
        "playToolBarArea",
        "configWindowGeometryChanged",
        "configWindowGeometry"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'fileListSortColumnChanged'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'fileListSortOrderChanged'
        QtMocHelpers::SignalData<void(Qt::SortOrder)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'fileListVisibleColumnsChanged'
        QtMocHelpers::SignalData<void(const QList<int> &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Signal 'fileListCustomColumnWidthsEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Signal 'fileListColumnWidthsChanged'
        QtMocHelpers::SignalData<void(const QList<int> &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 13 },
        }}),
        // Signal 'dirListSortColumnChanged'
        QtMocHelpers::SignalData<void(int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 },
        }}),
        // Signal 'dirListSortOrderChanged'
        QtMocHelpers::SignalData<void(Qt::SortOrder)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 17 },
        }}),
        // Signal 'dirListVisibleColumnsChanged'
        QtMocHelpers::SignalData<void(const QList<int> &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 19 },
        }}),
        // Signal 'dirListCustomColumnWidthsEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 21 },
        }}),
        // Signal 'dirListColumnWidthsChanged'
        QtMocHelpers::SignalData<void(const QList<int> &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 23 },
        }}),
        // Signal 'splitterSizesChanged'
        QtMocHelpers::SignalData<void(const QList<int> &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 25 },
        }}),
        // Signal 'vSplitterSizesChanged'
        QtMocHelpers::SignalData<void(const QList<int> &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 27 },
        }}),
        // Signal 'autoHideTagsChanged'
        QtMocHelpers::SignalData<void(bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 29 },
        }}),
        // Signal 'hideFileChanged'
        QtMocHelpers::SignalData<void(bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 31 },
        }}),
        // Signal 'hideTagChanged'
        QtMocHelpers::SignalData<void()>(32, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'hidePictureChanged'
        QtMocHelpers::SignalData<void(bool)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 34 },
        }}),
        // Signal 'playOnDoubleClickChanged'
        QtMocHelpers::SignalData<void(bool)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 36 },
        }}),
        // Signal 'selectFileOnPlayEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 38 },
        }}),
        // Signal 'playToolBarVisibleChanged'
        QtMocHelpers::SignalData<void(bool)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 40 },
        }}),
        // Signal 'preferredAudioOutputChanged'
        QtMocHelpers::SignalData<void(const QString &)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 42 },
        }}),
        // Signal 'playToolBarAreaChanged'
        QtMocHelpers::SignalData<void(int)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 44 },
        }}),
        // Signal 'configWindowGeometryChanged'
        QtMocHelpers::SignalData<void(const QByteArray &)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 46 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'fileListSortColumn'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'fileListSortOrder'
        QtMocHelpers::PropertyData<Qt::SortOrder>(6, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 1),
        // property 'fileListVisibleColumns'
        QtMocHelpers::PropertyData<QList<int>>(9, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'fileListCustomColumnWidthsEnabled'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'fileListColumnWidths'
        QtMocHelpers::PropertyData<QList<int>>(13, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 4),
        // property 'dirListSortColumn'
        QtMocHelpers::PropertyData<int>(15, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'dirListSortOrder'
        QtMocHelpers::PropertyData<Qt::SortOrder>(17, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 6),
        // property 'dirListVisibleColumns'
        QtMocHelpers::PropertyData<QList<int>>(19, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 7),
        // property 'dirListCustomColumnWidthsEnabled'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'dirListColumnWidths'
        QtMocHelpers::PropertyData<QList<int>>(23, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 9),
        // property 'splitterSizes'
        QtMocHelpers::PropertyData<QList<int>>(25, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 10),
        // property 'vSplitterSizes'
        QtMocHelpers::PropertyData<QList<int>>(27, 0x80000000 | 8, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 11),
        // property 'autoHideTags'
        QtMocHelpers::PropertyData<bool>(29, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'hideFile'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'hidePicture'
        QtMocHelpers::PropertyData<bool>(34, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'playOnDoubleClick'
        QtMocHelpers::PropertyData<bool>(36, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 16),
        // property 'selectFileOnPlayEnabled'
        QtMocHelpers::PropertyData<bool>(38, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 17),
        // property 'playToolBarVisible'
        QtMocHelpers::PropertyData<bool>(40, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 18),
        // property 'preferredAudioOutput'
        QtMocHelpers::PropertyData<QString>(42, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 19),
        // property 'playToolBarArea'
        QtMocHelpers::PropertyData<int>(44, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 20),
        // property 'configWindowGeometry'
        QtMocHelpers::PropertyData<QByteArray>(46, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 21),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<GuiConfig, qt_meta_tag_ZN9GuiConfigE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject GuiConfig::staticMetaObject = { {
    QMetaObject::SuperData::link<StoredConfig<GuiConfig>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9GuiConfigE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9GuiConfigE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9GuiConfigE_t>.metaTypes,
    nullptr
} };

void GuiConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<GuiConfig *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->fileListSortColumnChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->fileListSortOrderChanged((*reinterpret_cast<std::add_pointer_t<Qt::SortOrder>>(_a[1]))); break;
        case 2: _t->fileListVisibleColumnsChanged((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1]))); break;
        case 3: _t->fileListCustomColumnWidthsEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->fileListColumnWidthsChanged((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1]))); break;
        case 5: _t->dirListSortColumnChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->dirListSortOrderChanged((*reinterpret_cast<std::add_pointer_t<Qt::SortOrder>>(_a[1]))); break;
        case 7: _t->dirListVisibleColumnsChanged((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1]))); break;
        case 8: _t->dirListCustomColumnWidthsEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->dirListColumnWidthsChanged((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1]))); break;
        case 10: _t->splitterSizesChanged((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1]))); break;
        case 11: _t->vSplitterSizesChanged((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1]))); break;
        case 12: _t->autoHideTagsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->hideFileChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->hideTagChanged(); break;
        case 15: _t->hidePictureChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->playOnDoubleClickChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 17: _t->selectFileOnPlayEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 18: _t->playToolBarVisibleChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: _t->preferredAudioOutputChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 20: _t->playToolBarAreaChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 21: _t->configWindowGeometryChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 4:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)(int )>(_a, &GuiConfig::fileListSortColumnChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)(Qt::SortOrder )>(_a, &GuiConfig::fileListSortOrderChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)(const QList<int> & )>(_a, &GuiConfig::fileListVisibleColumnsChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)(bool )>(_a, &GuiConfig::fileListCustomColumnWidthsEnabledChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)(const QList<int> & )>(_a, &GuiConfig::fileListColumnWidthsChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)(int )>(_a, &GuiConfig::dirListSortColumnChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)(Qt::SortOrder )>(_a, &GuiConfig::dirListSortOrderChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)(const QList<int> & )>(_a, &GuiConfig::dirListVisibleColumnsChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)(bool )>(_a, &GuiConfig::dirListCustomColumnWidthsEnabledChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)(const QList<int> & )>(_a, &GuiConfig::dirListColumnWidthsChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)(const QList<int> & )>(_a, &GuiConfig::splitterSizesChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)(const QList<int> & )>(_a, &GuiConfig::vSplitterSizesChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)(bool )>(_a, &GuiConfig::autoHideTagsChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)(bool )>(_a, &GuiConfig::hideFileChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)()>(_a, &GuiConfig::hideTagChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)(bool )>(_a, &GuiConfig::hidePictureChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)(bool )>(_a, &GuiConfig::playOnDoubleClickChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)(bool )>(_a, &GuiConfig::selectFileOnPlayEnabledChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)(bool )>(_a, &GuiConfig::playToolBarVisibleChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)(const QString & )>(_a, &GuiConfig::preferredAudioOutputChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)(int )>(_a, &GuiConfig::playToolBarAreaChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (GuiConfig::*)(const QByteArray & )>(_a, &GuiConfig::configWindowGeometryChanged, 21))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 11:
        case 10:
        case 9:
        case 7:
        case 4:
        case 2:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->fileListSortColumn(); break;
        case 1: *reinterpret_cast<Qt::SortOrder*>(_v) = _t->fileListSortOrder(); break;
        case 2: *reinterpret_cast<QList<int>*>(_v) = _t->fileListVisibleColumns(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->fileListCustomColumnWidthsEnabled(); break;
        case 4: *reinterpret_cast<QList<int>*>(_v) = _t->fileListColumnWidths(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->dirListSortColumn(); break;
        case 6: *reinterpret_cast<Qt::SortOrder*>(_v) = _t->dirListSortOrder(); break;
        case 7: *reinterpret_cast<QList<int>*>(_v) = _t->dirListVisibleColumns(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->dirListCustomColumnWidthsEnabled(); break;
        case 9: *reinterpret_cast<QList<int>*>(_v) = _t->dirListColumnWidths(); break;
        case 10: *reinterpret_cast<QList<int>*>(_v) = _t->splitterSizes(); break;
        case 11: *reinterpret_cast<QList<int>*>(_v) = _t->vSplitterSizes(); break;
        case 12: *reinterpret_cast<bool*>(_v) = _t->autoHideTags(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->hideFile(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->hidePicture(); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->playOnDoubleClick(); break;
        case 16: *reinterpret_cast<bool*>(_v) = _t->selectFileOnPlayEnabled(); break;
        case 17: *reinterpret_cast<bool*>(_v) = _t->playToolBarVisible(); break;
        case 18: *reinterpret_cast<QString*>(_v) = _t->preferredAudioOutput(); break;
        case 19: *reinterpret_cast<int*>(_v) = _t->playToolBarArea(); break;
        case 20: *reinterpret_cast<QByteArray*>(_v) = _t->configWindowGeometry(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setFileListSortColumn(*reinterpret_cast<int*>(_v)); break;
        case 1: _t->setFileListSortOrder(*reinterpret_cast<Qt::SortOrder*>(_v)); break;
        case 2: _t->setFileListVisibleColumns(*reinterpret_cast<QList<int>*>(_v)); break;
        case 3: _t->setFileListCustomColumnWidthsEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setFileListColumnWidths(*reinterpret_cast<QList<int>*>(_v)); break;
        case 5: _t->setDirListSortColumn(*reinterpret_cast<int*>(_v)); break;
        case 6: _t->setDirListSortOrder(*reinterpret_cast<Qt::SortOrder*>(_v)); break;
        case 7: _t->setDirListVisibleColumns(*reinterpret_cast<QList<int>*>(_v)); break;
        case 8: _t->setDirListCustomColumnWidthsEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 9: _t->setDirListColumnWidths(*reinterpret_cast<QList<int>*>(_v)); break;
        case 10: _t->setSplitterSizes(*reinterpret_cast<QList<int>*>(_v)); break;
        case 11: _t->setVSplitterSizes(*reinterpret_cast<QList<int>*>(_v)); break;
        case 12: _t->setAutoHideTags(*reinterpret_cast<bool*>(_v)); break;
        case 13: _t->setHideFile(*reinterpret_cast<bool*>(_v)); break;
        case 14: _t->setHidePicture(*reinterpret_cast<bool*>(_v)); break;
        case 15: _t->setPlayOnDoubleClick(*reinterpret_cast<bool*>(_v)); break;
        case 16: _t->setSelectFileOnPlayEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 17: _t->setPlayToolBarVisible(*reinterpret_cast<bool*>(_v)); break;
        case 18: _t->setPreferredAudioOutput(*reinterpret_cast<QString*>(_v)); break;
        case 19: _t->setPlayToolBarArea(*reinterpret_cast<int*>(_v)); break;
        case 20: _t->setConfigWindowGeometry(*reinterpret_cast<QByteArray*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *GuiConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GuiConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9GuiConfigE_t>.strings))
        return static_cast<void*>(this);
    return StoredConfig<GuiConfig>::qt_metacast(_clname);
}

int GuiConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StoredConfig<GuiConfig>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void GuiConfig::fileListSortColumnChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void GuiConfig::fileListSortOrderChanged(Qt::SortOrder _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void GuiConfig::fileListVisibleColumnsChanged(const QList<int> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void GuiConfig::fileListCustomColumnWidthsEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void GuiConfig::fileListColumnWidthsChanged(const QList<int> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void GuiConfig::dirListSortColumnChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void GuiConfig::dirListSortOrderChanged(Qt::SortOrder _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void GuiConfig::dirListVisibleColumnsChanged(const QList<int> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void GuiConfig::dirListCustomColumnWidthsEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void GuiConfig::dirListColumnWidthsChanged(const QList<int> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void GuiConfig::splitterSizesChanged(const QList<int> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void GuiConfig::vSplitterSizesChanged(const QList<int> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void GuiConfig::autoHideTagsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void GuiConfig::hideFileChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void GuiConfig::hideTagChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 14, nullptr);
}

// SIGNAL 15
void GuiConfig::hidePictureChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void GuiConfig::playOnDoubleClickChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void GuiConfig::selectFileOnPlayEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void GuiConfig::playToolBarVisibleChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void GuiConfig::preferredAudioOutputChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void GuiConfig::playToolBarAreaChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void GuiConfig::configWindowGeometryChanged(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}
QT_WARNING_POP
