/****************************************************************************
** Meta object code from reading C++ file 'formatconfig.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/core/config/formatconfig.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'formatconfig.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12FormatConfigE_t {};
} // unnamed namespace

template <> constexpr inline auto FormatConfig::qt_create_metaobjectdata<qt_meta_tag_ZN12FormatConfigE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FormatConfig",
        "strRepMapChanged",
        "",
        "QList<std::pair<QString,QString>>",
        "strRepMap",
        "caseConversionChanged",
        "FormatConfig::CaseConversion",
        "caseConversion",
        "localeNameChanged",
        "localeName",
        "formatWhileEditingChanged",
        "formatWhileEditing",
        "strRepEnabledChanged",
        "strRepEnabled",
        "enableValidationChanged",
        "enableValidation",
        "useForOtherFileNamesChanged",
        "useForOtherFileNames",
        "enableMaximumLengthChanged",
        "enableMaximumLength",
        "maximumLengthChanged",
        "maximumLength",
        "getCaseConversionNames",
        "getLocaleNames",
        "CaseConversion",
        "NoChanges",
        "AllLowercase",
        "AllUppercase",
        "FirstLetterUppercase",
        "AllFirstLettersUppercase",
        "NumCaseConversions"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'strRepMapChanged'
        QtMocHelpers::SignalData<void(const QList<QPair<QString,QString>> &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'caseConversionChanged'
        QtMocHelpers::SignalData<void(FormatConfig::CaseConversion)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'localeNameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'formatWhileEditingChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Signal 'strRepEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Signal 'enableValidationChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Signal 'useForOtherFileNamesChanged'
        QtMocHelpers::SignalData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Signal 'enableMaximumLengthChanged'
        QtMocHelpers::SignalData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Signal 'maximumLengthChanged'
        QtMocHelpers::SignalData<void(int)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 21 },
        }}),
        // Method 'getCaseConversionNames'
        QtMocHelpers::MethodData<QStringList()>(22, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'getLocaleNames'
        QtMocHelpers::MethodData<QStringList()>(23, 2, QMC::AccessPublic, QMetaType::QStringList),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'strRepMap'
        QtMocHelpers::PropertyData<QStringList>(4, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'caseConversion'
        QtMocHelpers::PropertyData<int>(7, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable, 1),
        // property 'localeName'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'maximumLength'
        QtMocHelpers::PropertyData<int>(21, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'enableMaximumLength'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'formatWhileEditing'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'strRepEnabled'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'enableValidation'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'useForOtherFileNames'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'CaseConversion'
        QtMocHelpers::EnumData<enum CaseConversion>(24, 24, QMC::EnumFlags{}).add({
            {   25, CaseConversion::NoChanges },
            {   26, CaseConversion::AllLowercase },
            {   27, CaseConversion::AllUppercase },
            {   28, CaseConversion::FirstLetterUppercase },
            {   29, CaseConversion::AllFirstLettersUppercase },
            {   30, CaseConversion::NumCaseConversions },
        }),
    };
    return QtMocHelpers::metaObjectData<FormatConfig, qt_meta_tag_ZN12FormatConfigE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FormatConfig::staticMetaObject = { {
    QMetaObject::SuperData::link<GeneralConfig::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12FormatConfigE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12FormatConfigE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12FormatConfigE_t>.metaTypes,
    nullptr
} };

void FormatConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FormatConfig *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->strRepMapChanged((*reinterpret_cast<std::add_pointer_t<QList<std::pair<QString,QString>>>>(_a[1]))); break;
        case 1: _t->caseConversionChanged((*reinterpret_cast<std::add_pointer_t<FormatConfig::CaseConversion>>(_a[1]))); break;
        case 2: _t->localeNameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->formatWhileEditingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->strRepEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->enableValidationChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->useForOtherFileNamesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->enableMaximumLengthChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->maximumLengthChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 9: { QStringList _r = _t->getCaseConversionNames();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 10: { QStringList _r = _t->getLocaleNames();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FormatConfig::*)(const QList<QPair<QString,QString>> & )>(_a, &FormatConfig::strRepMapChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormatConfig::*)(FormatConfig::CaseConversion )>(_a, &FormatConfig::caseConversionChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormatConfig::*)(const QString & )>(_a, &FormatConfig::localeNameChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormatConfig::*)(bool )>(_a, &FormatConfig::formatWhileEditingChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormatConfig::*)(bool )>(_a, &FormatConfig::strRepEnabledChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormatConfig::*)(bool )>(_a, &FormatConfig::enableValidationChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormatConfig::*)(bool )>(_a, &FormatConfig::useForOtherFileNamesChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormatConfig::*)(bool )>(_a, &FormatConfig::enableMaximumLengthChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (FormatConfig::*)(int )>(_a, &FormatConfig::maximumLengthChanged, 8))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QStringList*>(_v) = _t->strRepStringList(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->caseConversion(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->localeName(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->maximumLength(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->enableMaximumLength(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->formatWhileEditing(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->strRepEnabled(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->enableValidation(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->useForOtherFileNames(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setStrRepStringList(*reinterpret_cast<QStringList*>(_v)); break;
        case 1: _t->setCaseConversionInt(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setLocaleName(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setMaximumLength(*reinterpret_cast<int*>(_v)); break;
        case 4: _t->setEnableMaximumLength(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setFormatWhileEditing(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setStrRepEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setEnableValidation(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setUseForOtherFileNames(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *FormatConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FormatConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12FormatConfigE_t>.strings))
        return static_cast<void*>(this);
    return GeneralConfig::qt_metacast(_clname);
}

int FormatConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = GeneralConfig::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void FormatConfig::strRepMapChanged(const QList<QPair<QString,QString>> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void FormatConfig::caseConversionChanged(FormatConfig::CaseConversion _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void FormatConfig::localeNameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void FormatConfig::formatWhileEditingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void FormatConfig::strRepEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void FormatConfig::enableValidationChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void FormatConfig::useForOtherFileNamesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void FormatConfig::enableMaximumLengthChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void FormatConfig::maximumLengthChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}
namespace {
struct qt_meta_tag_ZN20FilenameFormatConfigE_t {};
} // unnamed namespace

template <> constexpr inline auto FilenameFormatConfig::qt_create_metaobjectdata<qt_meta_tag_ZN20FilenameFormatConfigE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FilenameFormatConfig"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FilenameFormatConfig, qt_meta_tag_ZN20FilenameFormatConfigE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FilenameFormatConfig::staticMetaObject = { {
    QMetaObject::SuperData::link<StoredConfig<FilenameFormatConfig,FormatConfig>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20FilenameFormatConfigE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20FilenameFormatConfigE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20FilenameFormatConfigE_t>.metaTypes,
    nullptr
} };

void FilenameFormatConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FilenameFormatConfig *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *FilenameFormatConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FilenameFormatConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20FilenameFormatConfigE_t>.strings))
        return static_cast<void*>(this);
    return StoredConfig<FilenameFormatConfig,FormatConfig>::qt_metacast(_clname);
}

int FilenameFormatConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StoredConfig<FilenameFormatConfig,FormatConfig>::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN15TagFormatConfigE_t {};
} // unnamed namespace

template <> constexpr inline auto TagFormatConfig::qt_create_metaobjectdata<qt_meta_tag_ZN15TagFormatConfigE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TagFormatConfig"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TagFormatConfig, qt_meta_tag_ZN15TagFormatConfigE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TagFormatConfig::staticMetaObject = { {
    QMetaObject::SuperData::link<StoredConfig<TagFormatConfig,FormatConfig>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15TagFormatConfigE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15TagFormatConfigE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15TagFormatConfigE_t>.metaTypes,
    nullptr
} };

void TagFormatConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TagFormatConfig *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *TagFormatConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TagFormatConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15TagFormatConfigE_t>.strings))
        return static_cast<void*>(this);
    return StoredConfig<TagFormatConfig,FormatConfig>::qt_metacast(_clname);
}

int TagFormatConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StoredConfig<TagFormatConfig,FormatConfig>::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
