/****************************************************************************
** Meta object code from reading C++ file 'configobjects.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/plugins/kid3qml/configobjects.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'configobjects.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13ConfigObjectsE_t {};
} // unnamed namespace

template <> constexpr inline auto ConfigObjects::qt_create_metaobjectdata<qt_meta_tag_ZN13ConfigObjectsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ConfigObjects",
        "batchImportConfig",
        "",
        "filterConfig",
        "filenameFormatConfig",
        "tagFormatConfig",
        "importConfig",
        "exportConfig",
        "tagConfig",
        "fileConfig",
        "renDirConfig",
        "numberTracksConfig",
        "userActionsConfig",
        "guiConfig",
        "networkConfig",
        "playlistConfig",
        "findReplaceConfig",
        "mainWindowConfig"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'batchImportConfig'
        QtMocHelpers::MethodData<QObject *()>(1, 2, QMC::AccessPublic, QMetaType::QObjectStar),
        // Method 'filterConfig'
        QtMocHelpers::MethodData<QObject *()>(3, 2, QMC::AccessPublic, QMetaType::QObjectStar),
        // Method 'filenameFormatConfig'
        QtMocHelpers::MethodData<QObject *()>(4, 2, QMC::AccessPublic, QMetaType::QObjectStar),
        // Method 'tagFormatConfig'
        QtMocHelpers::MethodData<QObject *()>(5, 2, QMC::AccessPublic, QMetaType::QObjectStar),
        // Method 'importConfig'
        QtMocHelpers::MethodData<QObject *()>(6, 2, QMC::AccessPublic, QMetaType::QObjectStar),
        // Method 'exportConfig'
        QtMocHelpers::MethodData<QObject *()>(7, 2, QMC::AccessPublic, QMetaType::QObjectStar),
        // Method 'tagConfig'
        QtMocHelpers::MethodData<QObject *()>(8, 2, QMC::AccessPublic, QMetaType::QObjectStar),
        // Method 'fileConfig'
        QtMocHelpers::MethodData<QObject *()>(9, 2, QMC::AccessPublic, QMetaType::QObjectStar),
        // Method 'renDirConfig'
        QtMocHelpers::MethodData<QObject *()>(10, 2, QMC::AccessPublic, QMetaType::QObjectStar),
        // Method 'numberTracksConfig'
        QtMocHelpers::MethodData<QObject *()>(11, 2, QMC::AccessPublic, QMetaType::QObjectStar),
        // Method 'userActionsConfig'
        QtMocHelpers::MethodData<QObject *()>(12, 2, QMC::AccessPublic, QMetaType::QObjectStar),
        // Method 'guiConfig'
        QtMocHelpers::MethodData<QObject *()>(13, 2, QMC::AccessPublic, QMetaType::QObjectStar),
        // Method 'networkConfig'
        QtMocHelpers::MethodData<QObject *()>(14, 2, QMC::AccessPublic, QMetaType::QObjectStar),
        // Method 'playlistConfig'
        QtMocHelpers::MethodData<QObject *()>(15, 2, QMC::AccessPublic, QMetaType::QObjectStar),
        // Method 'findReplaceConfig'
        QtMocHelpers::MethodData<QObject *()>(16, 2, QMC::AccessPublic, QMetaType::QObjectStar),
        // Method 'mainWindowConfig'
        QtMocHelpers::MethodData<QObject *()>(17, 2, QMC::AccessPublic, QMetaType::QObjectStar),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ConfigObjects, qt_meta_tag_ZN13ConfigObjectsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ConfigObjects::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ConfigObjectsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ConfigObjectsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13ConfigObjectsE_t>.metaTypes,
    nullptr
} };

void ConfigObjects::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ConfigObjects *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QObject* _r = _t->batchImportConfig();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 1: { QObject* _r = _t->filterConfig();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 2: { QObject* _r = _t->filenameFormatConfig();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 3: { QObject* _r = _t->tagFormatConfig();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 4: { QObject* _r = _t->importConfig();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 5: { QObject* _r = _t->exportConfig();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 6: { QObject* _r = _t->tagConfig();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 7: { QObject* _r = _t->fileConfig();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 8: { QObject* _r = _t->renDirConfig();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 9: { QObject* _r = _t->numberTracksConfig();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 10: { QObject* _r = _t->userActionsConfig();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 11: { QObject* _r = _t->guiConfig();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 12: { QObject* _r = _t->networkConfig();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 13: { QObject* _r = _t->playlistConfig();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 14: { QObject* _r = _t->findReplaceConfig();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        case 15: { QObject* _r = _t->mainWindowConfig();
            if (_a[0]) *reinterpret_cast<QObject**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
}

const QMetaObject *ConfigObjects::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ConfigObjects::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ConfigObjectsE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ConfigObjects::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 16;
    }
    return _id;
}
QT_WARNING_POP
