/****************************************************************************
** Meta object code from reading C++ file 'qmlcommandplugin.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/plugins/qmlcommand/qmlcommandplugin.h"
#include <QtCore/qmetatype.h>
#include <QtCore/qplugin.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qmlcommandplugin.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16QmlCommandPluginE_t {};
} // unnamed namespace

template <> constexpr inline auto QmlCommandPlugin::qt_create_metaobjectdata<qt_meta_tag_ZN16QmlCommandPluginE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QmlCommandPlugin",
        "commandOutput",
        "",
        "msg",
        "finished",
        "exitCode",
        "onEngineError",
        "QList<QQmlError>",
        "errors",
        "onQmlViewClosing",
        "onQmlViewFinished",
        "onQmlEngineQuit",
        "onEngineFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'commandOutput'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'finished'
        QtMocHelpers::SignalData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Slot 'onEngineError'
        QtMocHelpers::SlotData<void(const QList<QQmlError> &)>(6, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Slot 'onQmlViewClosing'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onQmlViewFinished'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onQmlEngineQuit'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onEngineFinished'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QmlCommandPlugin, qt_meta_tag_ZN16QmlCommandPluginE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QmlCommandPlugin::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QmlCommandPluginE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QmlCommandPluginE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16QmlCommandPluginE_t>.metaTypes,
    nullptr
} };

void QmlCommandPlugin::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QmlCommandPlugin *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->commandOutput((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->finished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->onEngineError((*reinterpret_cast<std::add_pointer_t<QList<QQmlError>>>(_a[1]))); break;
        case 3: _t->onQmlViewClosing(); break;
        case 4: _t->onQmlViewFinished(); break;
        case 5: _t->onQmlEngineQuit(); break;
        case 6: _t->onEngineFinished(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QmlCommandPlugin::*)(const QString & )>(_a, &QmlCommandPlugin::commandOutput, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QmlCommandPlugin::*)(int )>(_a, &QmlCommandPlugin::finished, 1))
            return;
    }
}

const QMetaObject *QmlCommandPlugin::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QmlCommandPlugin::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16QmlCommandPluginE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "IUserCommandProcessor"))
        return static_cast< IUserCommandProcessor*>(this);
    if (!strcmp(_clname, "org.kde.kid3.IUserCommandProcessor"))
        return static_cast< IUserCommandProcessor*>(this);
    return QObject::qt_metacast(_clname);
}

int QmlCommandPlugin::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void QmlCommandPlugin::commandOutput(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QmlCommandPlugin::finished(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

#ifdef QT_MOC_EXPORT_PLUGIN_V2
static constexpr unsigned char qt_pluginMetaDataV2_QmlCommandPlugin[] = {
    0xbf, 
    // "IID"
    0x02,  0x78,  0x22,  'o',  'r',  'g',  '.',  'k', 
    'd',  'e',  '.',  'k',  'i',  'd',  '3',  '.', 
    'I',  'U',  's',  'e',  'r',  'C',  'o',  'm', 
    'm',  'a',  'n',  'd',  'P',  'r',  'o',  'c', 
    'e',  's',  's',  'o',  'r', 
    // "className"
    0x03,  0x70,  'Q',  'm',  'l',  'C',  'o',  'm', 
    'm',  'a',  'n',  'd',  'P',  'l',  'u',  'g', 
    'i',  'n', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN_V2(QmlCommandPlugin, QmlCommandPlugin, qt_pluginMetaDataV2_QmlCommandPlugin)
#else
QT_PLUGIN_METADATA_SECTION
Q_CONSTINIT static constexpr unsigned char qt_pluginMetaData_QmlCommandPlugin[] = {
    'Q', 'T', 'M', 'E', 'T', 'A', 'D', 'A', 'T', 'A', ' ', '!',
    // metadata version, Qt version, architectural requirements
    0, QT_VERSION_MAJOR, QT_VERSION_MINOR, qPluginArchRequirements(),
    0xbf, 
    // "IID"
    0x02,  0x78,  0x22,  'o',  'r',  'g',  '.',  'k', 
    'd',  'e',  '.',  'k',  'i',  'd',  '3',  '.', 
    'I',  'U',  's',  'e',  'r',  'C',  'o',  'm', 
    'm',  'a',  'n',  'd',  'P',  'r',  'o',  'c', 
    'e',  's',  's',  'o',  'r', 
    // "className"
    0x03,  0x70,  'Q',  'm',  'l',  'C',  'o',  'm', 
    'm',  'a',  'n',  'd',  'P',  'l',  'u',  'g', 
    'i',  'n', 
    0xff, 
};
QT_MOC_EXPORT_PLUGIN(QmlCommandPlugin, QmlCommandPlugin)
#endif  // QT_MOC_EXPORT_PLUGIN_V2

QT_WARNING_POP
