/****************************************************************************
** Meta object code from reading C++ file 'importconfig.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/core/config/importconfig.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'importconfig.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12ImportConfigE_t {};
} // unnamed namespace

template <> constexpr inline auto ImportConfig::qt_create_metaobjectdata<qt_meta_tag_ZN12ImportConfigE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ImportConfig",
        "availablePluginsChanged",
        "",
        "availablePlugins",
        "importServerChanged",
        "importServer",
        "importDestChanged",
        "Frame::TagVersion",
        "importDest",
        "importFormatNamesChanged",
        "importFormatNames",
        "importFormatHeadersChanged",
        "importFormatHeaders",
        "importFormatTracksChanged",
        "importFormatTracks",
        "importFormatIndexChanged",
        "importFormatIndex",
        "maxTimeDifferenceChanged",
        "maxTimeDifference",
        "importVisibleColumnsChanged",
        "importVisibleColumns",
        "importWindowGeometryChanged",
        "importWindowGeometry",
        "importTagsNamesChanged",
        "importTagsNames",
        "importTagsSourcesChanged",
        "importTagsSources",
        "importTagsExtractionsChanged",
        "importTagsExtractions",
        "importTagsIndexChanged",
        "importTagsIndex",
        "pictureSourceNamesChanged",
        "pictureSourceNames",
        "pictureSourceUrlsChanged",
        "pictureSourceUrls",
        "pictureSourceIndexChanged",
        "pictureSourceIndex",
        "browseCoverArtWindowGeometryChanged",
        "browseCoverArtWindowGeometry",
        "matchPictureUrlMapChanged",
        "QList<std::pair<QString,QString>>",
        "matchPictureUrlMap",
        "importDirChanged",
        "importDir",
        "disabledPluginsChanged",
        "disabledPlugins",
        "enableTimeDifferenceCheckChanged",
        "enableTimeDifferenceCheck"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'availablePluginsChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 3 },
        }}),
        // Signal 'importServerChanged'
        QtMocHelpers::SignalData<void(int)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 5 },
        }}),
        // Signal 'importDestChanged'
        QtMocHelpers::SignalData<void(Frame::TagVersion)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'importFormatNamesChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 10 },
        }}),
        // Signal 'importFormatHeadersChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 12 },
        }}),
        // Signal 'importFormatTracksChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 14 },
        }}),
        // Signal 'importFormatIndexChanged'
        QtMocHelpers::SignalData<void(int)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 16 },
        }}),
        // Signal 'maxTimeDifferenceChanged'
        QtMocHelpers::SignalData<void(int)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 18 },
        }}),
        // Signal 'importVisibleColumnsChanged'
        QtMocHelpers::SignalData<void(quint64)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 20 },
        }}),
        // Signal 'importWindowGeometryChanged'
        QtMocHelpers::SignalData<void(const QByteArray &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 22 },
        }}),
        // Signal 'importTagsNamesChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 24 },
        }}),
        // Signal 'importTagsSourcesChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 26 },
        }}),
        // Signal 'importTagsExtractionsChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 28 },
        }}),
        // Signal 'importTagsIndexChanged'
        QtMocHelpers::SignalData<void(int)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 30 },
        }}),
        // Signal 'pictureSourceNamesChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 32 },
        }}),
        // Signal 'pictureSourceUrlsChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 34 },
        }}),
        // Signal 'pictureSourceIndexChanged'
        QtMocHelpers::SignalData<void(int)>(35, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 36 },
        }}),
        // Signal 'browseCoverArtWindowGeometryChanged'
        QtMocHelpers::SignalData<void(const QByteArray &)>(37, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 38 },
        }}),
        // Signal 'matchPictureUrlMapChanged'
        QtMocHelpers::SignalData<void(const QList<QPair<QString,QString>> &)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 40, 41 },
        }}),
        // Signal 'importDirChanged'
        QtMocHelpers::SignalData<void(const QString &)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 43 },
        }}),
        // Signal 'disabledPluginsChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(44, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 45 },
        }}),
        // Signal 'enableTimeDifferenceCheckChanged'
        QtMocHelpers::SignalData<void(bool)>(46, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 47 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'availablePlugins'
        QtMocHelpers::PropertyData<QStringList>(3, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'importServer'
        QtMocHelpers::PropertyData<int>(5, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'importDest'
        QtMocHelpers::PropertyData<int>(8, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable, 2),
        // property 'importFormatNames'
        QtMocHelpers::PropertyData<QStringList>(10, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'importFormatHeaders'
        QtMocHelpers::PropertyData<QStringList>(12, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'importFormatTracks'
        QtMocHelpers::PropertyData<QStringList>(14, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'importFormatIndex'
        QtMocHelpers::PropertyData<int>(16, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'maxTimeDifference'
        QtMocHelpers::PropertyData<int>(18, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'importVisibleColumns'
        QtMocHelpers::PropertyData<quint64>(20, QMetaType::ULongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'importWindowGeometry'
        QtMocHelpers::PropertyData<QByteArray>(22, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'importTagsNames'
        QtMocHelpers::PropertyData<QStringList>(24, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'importTagsSources'
        QtMocHelpers::PropertyData<QStringList>(26, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'importTagsExtractions'
        QtMocHelpers::PropertyData<QStringList>(28, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'importTagsIndex'
        QtMocHelpers::PropertyData<int>(30, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'pictureSourceNames'
        QtMocHelpers::PropertyData<QStringList>(32, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 14),
        // property 'pictureSourceUrls'
        QtMocHelpers::PropertyData<QStringList>(34, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'pictureSourceIndex'
        QtMocHelpers::PropertyData<int>(36, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 16),
        // property 'browseCoverArtWindowGeometry'
        QtMocHelpers::PropertyData<QByteArray>(38, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 17),
        // property 'matchPictureUrlMap'
        QtMocHelpers::PropertyData<QStringList>(41, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable, 18),
        // property 'importDir'
        QtMocHelpers::PropertyData<QString>(43, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 19),
        // property 'disabledPlugins'
        QtMocHelpers::PropertyData<QStringList>(45, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 20),
        // property 'enableTimeDifferenceCheck'
        QtMocHelpers::PropertyData<bool>(47, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 21),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ImportConfig, qt_meta_tag_ZN12ImportConfigE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ImportConfig::staticMetaObject = { {
    QMetaObject::SuperData::link<StoredConfig<ImportConfig>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12ImportConfigE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12ImportConfigE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12ImportConfigE_t>.metaTypes,
    nullptr
} };

void ImportConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ImportConfig *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->availablePluginsChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 1: _t->importServerChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->importDestChanged((*reinterpret_cast<std::add_pointer_t<Frame::TagVersion>>(_a[1]))); break;
        case 3: _t->importFormatNamesChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 4: _t->importFormatHeadersChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 5: _t->importFormatTracksChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 6: _t->importFormatIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 7: _t->maxTimeDifferenceChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 8: _t->importVisibleColumnsChanged((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1]))); break;
        case 9: _t->importWindowGeometryChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 10: _t->importTagsNamesChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 11: _t->importTagsSourcesChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 12: _t->importTagsExtractionsChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 13: _t->importTagsIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 14: _t->pictureSourceNamesChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 15: _t->pictureSourceUrlsChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 16: _t->pictureSourceIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->browseCoverArtWindowGeometryChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 18: _t->matchPictureUrlMapChanged((*reinterpret_cast<std::add_pointer_t<QList<std::pair<QString,QString>>>>(_a[1]))); break;
        case 19: _t->importDirChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 20: _t->disabledPluginsChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 21: _t->enableTimeDifferenceCheckChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(const QStringList & )>(_a, &ImportConfig::availablePluginsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(int )>(_a, &ImportConfig::importServerChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(Frame::TagVersion )>(_a, &ImportConfig::importDestChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(const QStringList & )>(_a, &ImportConfig::importFormatNamesChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(const QStringList & )>(_a, &ImportConfig::importFormatHeadersChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(const QStringList & )>(_a, &ImportConfig::importFormatTracksChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(int )>(_a, &ImportConfig::importFormatIndexChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(int )>(_a, &ImportConfig::maxTimeDifferenceChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(quint64 )>(_a, &ImportConfig::importVisibleColumnsChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(const QByteArray & )>(_a, &ImportConfig::importWindowGeometryChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(const QStringList & )>(_a, &ImportConfig::importTagsNamesChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(const QStringList & )>(_a, &ImportConfig::importTagsSourcesChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(const QStringList & )>(_a, &ImportConfig::importTagsExtractionsChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(int )>(_a, &ImportConfig::importTagsIndexChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(const QStringList & )>(_a, &ImportConfig::pictureSourceNamesChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(const QStringList & )>(_a, &ImportConfig::pictureSourceUrlsChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(int )>(_a, &ImportConfig::pictureSourceIndexChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(const QByteArray & )>(_a, &ImportConfig::browseCoverArtWindowGeometryChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(const QList<QPair<QString,QString>> & )>(_a, &ImportConfig::matchPictureUrlMapChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(const QString & )>(_a, &ImportConfig::importDirChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(const QStringList & )>(_a, &ImportConfig::disabledPluginsChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (ImportConfig::*)(bool )>(_a, &ImportConfig::enableTimeDifferenceCheckChanged, 21))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QStringList*>(_v) = _t->availablePlugins(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->importServer(); break;
        case 2: *reinterpret_cast<int*>(_v) = _t->importDest(); break;
        case 3: *reinterpret_cast<QStringList*>(_v) = _t->importFormatNames(); break;
        case 4: *reinterpret_cast<QStringList*>(_v) = _t->importFormatHeaders(); break;
        case 5: *reinterpret_cast<QStringList*>(_v) = _t->importFormatTracks(); break;
        case 6: *reinterpret_cast<int*>(_v) = _t->importFormatIndex(); break;
        case 7: *reinterpret_cast<int*>(_v) = _t->maxTimeDifference(); break;
        case 8: *reinterpret_cast<quint64*>(_v) = _t->importVisibleColumns(); break;
        case 9: *reinterpret_cast<QByteArray*>(_v) = _t->importWindowGeometry(); break;
        case 10: *reinterpret_cast<QStringList*>(_v) = _t->importTagsNames(); break;
        case 11: *reinterpret_cast<QStringList*>(_v) = _t->importTagsSources(); break;
        case 12: *reinterpret_cast<QStringList*>(_v) = _t->importTagsExtractions(); break;
        case 13: *reinterpret_cast<int*>(_v) = _t->importTagsIndex(); break;
        case 14: *reinterpret_cast<QStringList*>(_v) = _t->pictureSourceNames(); break;
        case 15: *reinterpret_cast<QStringList*>(_v) = _t->pictureSourceUrls(); break;
        case 16: *reinterpret_cast<int*>(_v) = _t->pictureSourceIndex(); break;
        case 17: *reinterpret_cast<QByteArray*>(_v) = _t->browseCoverArtWindowGeometry(); break;
        case 18: *reinterpret_cast<QStringList*>(_v) = _t->matchPictureUrlStringList(); break;
        case 19: *reinterpret_cast<QString*>(_v) = _t->importDir(); break;
        case 20: *reinterpret_cast<QStringList*>(_v) = _t->disabledPlugins(); break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->enableTimeDifferenceCheck(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAvailablePlugins(*reinterpret_cast<QStringList*>(_v)); break;
        case 1: _t->setImportServer(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setImportDestInt(*reinterpret_cast<int*>(_v)); break;
        case 3: _t->setImportFormatNames(*reinterpret_cast<QStringList*>(_v)); break;
        case 4: _t->setImportFormatHeaders(*reinterpret_cast<QStringList*>(_v)); break;
        case 5: _t->setImportFormatTracks(*reinterpret_cast<QStringList*>(_v)); break;
        case 6: _t->setImportFormatIndex(*reinterpret_cast<int*>(_v)); break;
        case 7: _t->setMaxTimeDifference(*reinterpret_cast<int*>(_v)); break;
        case 8: _t->setImportVisibleColumns(*reinterpret_cast<quint64*>(_v)); break;
        case 9: _t->setImportWindowGeometry(*reinterpret_cast<QByteArray*>(_v)); break;
        case 10: _t->setImportTagsNames(*reinterpret_cast<QStringList*>(_v)); break;
        case 11: _t->setImportTagsSources(*reinterpret_cast<QStringList*>(_v)); break;
        case 12: _t->setImportTagsExtractions(*reinterpret_cast<QStringList*>(_v)); break;
        case 13: _t->setImportTagsIndex(*reinterpret_cast<int*>(_v)); break;
        case 14: _t->setPictureSourceNames(*reinterpret_cast<QStringList*>(_v)); break;
        case 15: _t->setPictureSourceUrls(*reinterpret_cast<QStringList*>(_v)); break;
        case 16: _t->setPictureSourceIndex(*reinterpret_cast<int*>(_v)); break;
        case 17: _t->setBrowseCoverArtWindowGeometry(*reinterpret_cast<QByteArray*>(_v)); break;
        case 18: _t->setMatchPictureUrlStringList(*reinterpret_cast<QStringList*>(_v)); break;
        case 19: _t->setImportDir(*reinterpret_cast<QString*>(_v)); break;
        case 20: _t->setDisabledPlugins(*reinterpret_cast<QStringList*>(_v)); break;
        case 21: _t->setEnableTimeDifferenceCheck(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *ImportConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ImportConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12ImportConfigE_t>.strings))
        return static_cast<void*>(this);
    return StoredConfig<ImportConfig>::qt_metacast(_clname);
}

int ImportConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StoredConfig<ImportConfig>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 22;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    return _id;
}

// SIGNAL 0
void ImportConfig::availablePluginsChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void ImportConfig::importServerChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void ImportConfig::importDestChanged(Frame::TagVersion _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void ImportConfig::importFormatNamesChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void ImportConfig::importFormatHeadersChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void ImportConfig::importFormatTracksChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void ImportConfig::importFormatIndexChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void ImportConfig::maxTimeDifferenceChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void ImportConfig::importVisibleColumnsChanged(quint64 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void ImportConfig::importWindowGeometryChanged(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void ImportConfig::importTagsNamesChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void ImportConfig::importTagsSourcesChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void ImportConfig::importTagsExtractionsChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void ImportConfig::importTagsIndexChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void ImportConfig::pictureSourceNamesChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void ImportConfig::pictureSourceUrlsChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void ImportConfig::pictureSourceIndexChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void ImportConfig::browseCoverArtWindowGeometryChanged(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void ImportConfig::matchPictureUrlMapChanged(const QList<QPair<QString,QString>> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void ImportConfig::importDirChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void ImportConfig::disabledPluginsChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void ImportConfig::enableTimeDifferenceCheckChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}
QT_WARNING_POP
