/****************************************************************************
** Meta object code from reading C++ file 'tagconfig.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/core/config/tagconfig.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tagconfig.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN9TagConfigE_t {};
} // unnamed namespace

template <> constexpr inline auto TagConfig::qt_create_metaobjectdata<qt_meta_tag_ZN9TagConfigE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TagConfig",
        "taggedFileFeaturesChanged",
        "",
        "taggedFileFeatures",
        "markTruncationsChanged",
        "markTruncations",
        "maximumPictureSizeChanged",
        "maximumPictureSize",
        "markOversizedPicturesChanged",
        "markOversizedPictures",
        "markStandardViolationsChanged",
        "markStandardViolations",
        "enableTotalNumberOfTracksChanged",
        "enableTotalNumberOfTracks",
        "genreNotNumericChanged",
        "genreNotNumeric",
        "lowercaseId3RiffChunkChanged",
        "lowercaseId3RiffChunk",
        "commentNameChanged",
        "commentName",
        "pictureNameIndexChanged",
        "pictureNameIndex",
        "riffTrackNameChanged",
        "riffTrackName",
        "matroskaWriteStyleChanged",
        "matroskaWriteStyle",
        "customGenresChanged",
        "customGenres",
        "customFramesChanged",
        "customFrames",
        "id3v2VersionChanged",
        "id3v2Version",
        "textEncodingV1Changed",
        "textEncodingV1",
        "textEncodingChanged",
        "textEncoding",
        "quickAccessFramesChanged",
        "quickAccessFrames",
        "quickAccessFrameOrderChanged",
        "QList<int>",
        "frameTypes",
        "trackNumberDigitsChanged",
        "trackNumberDigits",
        "onlyCustomGenresChanged",
        "onlyCustomGenres",
        "pluginOrderChanged",
        "pluginOrder",
        "disabledPluginsChanged",
        "disabledPlugins",
        "availablePluginsChanged",
        "availablePlugins",
        "starRatingMappingsChanged",
        "selectedQuickAccessFrames",
        "QVariantList",
        "setSelectedQuickAccessFrames",
        "namesSelected",
        "starCountFromRating",
        "rating",
        "type",
        "starCountToRating",
        "starCount",
        "getTextEncodingNames",
        "getId3v2VersionNames",
        "getCommentNames",
        "getPictureNames",
        "getRiffTrackNames",
        "getMatroskaWriteStyles",
        "customFrameNamesToDisplayNames",
        "names",
        "customFrameNamesFromDisplayNames",
        "displayNames",
        "textEncodingV1Index",
        "quickAccessFrameOrder",
        "starRatingMappingStrings",
        "defaultPopmEmail",
        "Id3v2Version",
        "ID3v2_3_0",
        "ID3v2_4_0",
        "TextEncoding",
        "TE_ISO8859_1",
        "TE_UTF16",
        "TE_UTF8",
        "VorbisPictureName",
        "VP_METADATA_BLOCK_PICTURE",
        "VP_COVERART"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'taggedFileFeaturesChanged'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'markTruncationsChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Signal 'maximumPictureSizeChanged'
        QtMocHelpers::SignalData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Signal 'markOversizedPicturesChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'markStandardViolationsChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Signal 'enableTotalNumberOfTracksChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Signal 'genreNotNumericChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Signal 'lowercaseId3RiffChunkChanged'
        QtMocHelpers::SignalData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Signal 'commentNameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Signal 'pictureNameIndexChanged'
        QtMocHelpers::SignalData<void(int)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 21 },
        }}),
        // Signal 'riffTrackNameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Signal 'matroskaWriteStyleChanged'
        QtMocHelpers::SignalData<void(const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Signal 'customGenresChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 27 },
        }}),
        // Signal 'customFramesChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 29 },
        }}),
        // Signal 'id3v2VersionChanged'
        QtMocHelpers::SignalData<void(int)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 31 },
        }}),
        // Signal 'textEncodingV1Changed'
        QtMocHelpers::SignalData<void(const QString &)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 33 },
        }}),
        // Signal 'textEncodingChanged'
        QtMocHelpers::SignalData<void(int)>(34, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 35 },
        }}),
        // Signal 'quickAccessFramesChanged'
        QtMocHelpers::SignalData<void(quint64)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::ULongLong, 37 },
        }}),
        // Signal 'quickAccessFrameOrderChanged'
        QtMocHelpers::SignalData<void(const QList<int> &)>(38, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 39, 40 },
        }}),
        // Signal 'trackNumberDigitsChanged'
        QtMocHelpers::SignalData<void(int)>(41, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 42 },
        }}),
        // Signal 'onlyCustomGenresChanged'
        QtMocHelpers::SignalData<void(bool)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 44 },
        }}),
        // Signal 'pluginOrderChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(45, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 46 },
        }}),
        // Signal 'disabledPluginsChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(47, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 48 },
        }}),
        // Signal 'availablePluginsChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(49, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 50 },
        }}),
        // Signal 'starRatingMappingsChanged'
        QtMocHelpers::SignalData<void()>(51, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'selectedQuickAccessFrames'
        QtMocHelpers::MethodData<QVariantList() const>(52, 2, QMC::AccessPublic, 0x80000000 | 53),
        // Method 'setSelectedQuickAccessFrames'
        QtMocHelpers::MethodData<void(const QVariantList &)>(54, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 53, 55 },
        }}),
        // Method 'starCountFromRating'
        QtMocHelpers::MethodData<int(int, const QString &) const>(56, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::Int, 57 }, { QMetaType::QString, 58 },
        }}),
        // Method 'starCountToRating'
        QtMocHelpers::MethodData<int(int, const QString &) const>(59, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::Int, 60 }, { QMetaType::QString, 58 },
        }}),
        // Method 'getTextEncodingNames'
        QtMocHelpers::MethodData<QStringList()>(61, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'getId3v2VersionNames'
        QtMocHelpers::MethodData<QStringList()>(62, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'getCommentNames'
        QtMocHelpers::MethodData<QStringList()>(63, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'getPictureNames'
        QtMocHelpers::MethodData<QStringList()>(64, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'getRiffTrackNames'
        QtMocHelpers::MethodData<QStringList()>(65, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'getMatroskaWriteStyles'
        QtMocHelpers::MethodData<QStringList()>(66, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'customFrameNamesToDisplayNames'
        QtMocHelpers::MethodData<QStringList(const QStringList &)>(67, 2, QMC::AccessPublic, QMetaType::QStringList, {{
            { QMetaType::QStringList, 68 },
        }}),
        // Method 'customFrameNamesFromDisplayNames'
        QtMocHelpers::MethodData<QStringList(const QStringList &)>(69, 2, QMC::AccessPublic, QMetaType::QStringList, {{
            { QMetaType::QStringList, 70 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'taggedFileFeatures'
        QtMocHelpers::PropertyData<int>(3, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'markTruncations'
        QtMocHelpers::PropertyData<bool>(5, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'markOversizedPictures'
        QtMocHelpers::PropertyData<bool>(9, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'maximumPictureSize'
        QtMocHelpers::PropertyData<int>(7, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'markStandardViolations'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'enableTotalNumberOfTracks'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'genreNotNumeric'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'lowercaseId3RiffChunk'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'commentName'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'pictureNameIndex'
        QtMocHelpers::PropertyData<int>(21, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'riffTrackName'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'matroskaWriteStyle'
        QtMocHelpers::PropertyData<QString>(25, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'customGenres'
        QtMocHelpers::PropertyData<QStringList>(27, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'customFrames'
        QtMocHelpers::PropertyData<QStringList>(29, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'id3v2Version'
        QtMocHelpers::PropertyData<int>(31, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 14),
        // property 'textEncodingV1'
        QtMocHelpers::PropertyData<QString>(33, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'textEncodingV1Index'
        QtMocHelpers::PropertyData<int>(71, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 15),
        // property 'textEncoding'
        QtMocHelpers::PropertyData<int>(35, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 16),
        // property 'quickAccessFrames'
        QtMocHelpers::PropertyData<quint64>(37, QMetaType::ULongLong, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 17),
        // property 'quickAccessFrameOrder'
        QtMocHelpers::PropertyData<QList<int>>(72, 0x80000000 | 39, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 18),
        // property 'trackNumberDigits'
        QtMocHelpers::PropertyData<int>(42, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 19),
        // property 'onlyCustomGenres'
        QtMocHelpers::PropertyData<bool>(44, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 20),
        // property 'pluginOrder'
        QtMocHelpers::PropertyData<QStringList>(46, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 21),
        // property 'disabledPlugins'
        QtMocHelpers::PropertyData<QStringList>(48, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 22),
        // property 'availablePlugins'
        QtMocHelpers::PropertyData<QStringList>(50, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 23),
        // property 'starRatingMappingStrings'
        QtMocHelpers::PropertyData<QStringList>(73, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 24),
        // property 'defaultPopmEmail'
        QtMocHelpers::PropertyData<QString>(74, QMetaType::QString, QMC::DefaultPropertyFlags, 24),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Id3v2Version'
        QtMocHelpers::EnumData<enum Id3v2Version>(75, 75, QMC::EnumFlags{}).add({
            {   76, Id3v2Version::ID3v2_3_0 },
            {   77, Id3v2Version::ID3v2_4_0 },
        }),
        // enum 'TextEncoding'
        QtMocHelpers::EnumData<enum TextEncoding>(78, 78, QMC::EnumFlags{}).add({
            {   79, TextEncoding::TE_ISO8859_1 },
            {   80, TextEncoding::TE_UTF16 },
            {   81, TextEncoding::TE_UTF8 },
        }),
        // enum 'VorbisPictureName'
        QtMocHelpers::EnumData<enum VorbisPictureName>(82, 82, QMC::EnumFlags{}).add({
            {   83, VorbisPictureName::VP_METADATA_BLOCK_PICTURE },
            {   84, VorbisPictureName::VP_COVERART },
        }),
    };
    return QtMocHelpers::metaObjectData<TagConfig, qt_meta_tag_ZN9TagConfigE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TagConfig::staticMetaObject = { {
    QMetaObject::SuperData::link<StoredConfig<TagConfig>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9TagConfigE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9TagConfigE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9TagConfigE_t>.metaTypes,
    nullptr
} };

void TagConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TagConfig *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->taggedFileFeaturesChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->markTruncationsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->maximumPictureSizeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 3: _t->markOversizedPicturesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->markStandardViolationsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->enableTotalNumberOfTracksChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->genreNotNumericChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->lowercaseId3RiffChunkChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->commentNameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->pictureNameIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 10: _t->riffTrackNameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->matroskaWriteStyleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->customGenresChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 13: _t->customFramesChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 14: _t->id3v2VersionChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 15: _t->textEncodingV1Changed((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->textEncodingChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 17: _t->quickAccessFramesChanged((*reinterpret_cast<std::add_pointer_t<quint64>>(_a[1]))); break;
        case 18: _t->quickAccessFrameOrderChanged((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1]))); break;
        case 19: _t->trackNumberDigitsChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 20: _t->onlyCustomGenresChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 21: _t->pluginOrderChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 22: _t->disabledPluginsChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 23: _t->availablePluginsChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 24: _t->starRatingMappingsChanged(); break;
        case 25: { QVariantList _r = _t->selectedQuickAccessFrames();
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 26: _t->setSelectedQuickAccessFrames((*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[1]))); break;
        case 27: { int _r = _t->starCountFromRating((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 28: { int _r = _t->starCountToRating((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 29: { QStringList _r = _t->getTextEncodingNames();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 30: { QStringList _r = _t->getId3v2VersionNames();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 31: { QStringList _r = _t->getCommentNames();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 32: { QStringList _r = _t->getPictureNames();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 33: { QStringList _r = _t->getRiffTrackNames();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 34: { QStringList _r = _t->getMatroskaWriteStyles();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 35: { QStringList _r = _t->customFrameNamesToDisplayNames((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 36: { QStringList _r = _t->customFrameNamesFromDisplayNames((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 18:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(int )>(_a, &TagConfig::taggedFileFeaturesChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(bool )>(_a, &TagConfig::markTruncationsChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(int )>(_a, &TagConfig::maximumPictureSizeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(bool )>(_a, &TagConfig::markOversizedPicturesChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(bool )>(_a, &TagConfig::markStandardViolationsChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(bool )>(_a, &TagConfig::enableTotalNumberOfTracksChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(bool )>(_a, &TagConfig::genreNotNumericChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(bool )>(_a, &TagConfig::lowercaseId3RiffChunkChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(const QString & )>(_a, &TagConfig::commentNameChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(int )>(_a, &TagConfig::pictureNameIndexChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(const QString & )>(_a, &TagConfig::riffTrackNameChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(const QString & )>(_a, &TagConfig::matroskaWriteStyleChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(const QStringList & )>(_a, &TagConfig::customGenresChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(const QStringList & )>(_a, &TagConfig::customFramesChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(int )>(_a, &TagConfig::id3v2VersionChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(const QString & )>(_a, &TagConfig::textEncodingV1Changed, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(int )>(_a, &TagConfig::textEncodingChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(quint64 )>(_a, &TagConfig::quickAccessFramesChanged, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(const QList<int> & )>(_a, &TagConfig::quickAccessFrameOrderChanged, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(int )>(_a, &TagConfig::trackNumberDigitsChanged, 19))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(bool )>(_a, &TagConfig::onlyCustomGenresChanged, 20))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(const QStringList & )>(_a, &TagConfig::pluginOrderChanged, 21))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(const QStringList & )>(_a, &TagConfig::disabledPluginsChanged, 22))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)(const QStringList & )>(_a, &TagConfig::availablePluginsChanged, 23))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagConfig::*)()>(_a, &TagConfig::starRatingMappingsChanged, 24))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 19:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<int> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->taggedFileFeatures(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->markTruncations(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->markOversizedPictures(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->maximumPictureSize(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->markStandardViolations(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->enableTotalNumberOfTracks(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->genreNotNumeric(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->lowercaseId3RiffChunk(); break;
        case 8: *reinterpret_cast<QString*>(_v) = _t->commentName(); break;
        case 9: *reinterpret_cast<int*>(_v) = _t->pictureNameIndex(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->riffTrackName(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->matroskaWriteStyle(); break;
        case 12: *reinterpret_cast<QStringList*>(_v) = _t->customGenres(); break;
        case 13: *reinterpret_cast<QStringList*>(_v) = _t->customFrames(); break;
        case 14: *reinterpret_cast<int*>(_v) = _t->id3v2Version(); break;
        case 15: *reinterpret_cast<QString*>(_v) = _t->textEncodingV1(); break;
        case 16: *reinterpret_cast<int*>(_v) = _t->textEncodingV1Index(); break;
        case 17: *reinterpret_cast<int*>(_v) = _t->textEncoding(); break;
        case 18: *reinterpret_cast<quint64*>(_v) = _t->quickAccessFrames(); break;
        case 19: *reinterpret_cast<QList<int>*>(_v) = _t->quickAccessFrameOrder(); break;
        case 20: *reinterpret_cast<int*>(_v) = _t->trackNumberDigits(); break;
        case 21: *reinterpret_cast<bool*>(_v) = _t->onlyCustomGenres(); break;
        case 22: *reinterpret_cast<QStringList*>(_v) = _t->pluginOrder(); break;
        case 23: *reinterpret_cast<QStringList*>(_v) = _t->disabledPlugins(); break;
        case 24: *reinterpret_cast<QStringList*>(_v) = _t->availablePlugins(); break;
        case 25: *reinterpret_cast<QStringList*>(_v) = _t->starRatingMappingStrings(); break;
        case 26: *reinterpret_cast<QString*>(_v) = _t->defaultPopmEmail(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setTaggedFileFeatures(*reinterpret_cast<int*>(_v)); break;
        case 1: _t->setMarkTruncations(*reinterpret_cast<bool*>(_v)); break;
        case 2: _t->setMarkOversizedPictures(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setMaximumPictureSize(*reinterpret_cast<int*>(_v)); break;
        case 4: _t->setMarkStandardViolations(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setEnableTotalNumberOfTracks(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setGenreNotNumeric(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setLowercaseId3RiffChunk(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setCommentName(*reinterpret_cast<QString*>(_v)); break;
        case 9: _t->setPictureNameIndex(*reinterpret_cast<int*>(_v)); break;
        case 10: _t->setRiffTrackName(*reinterpret_cast<QString*>(_v)); break;
        case 11: _t->setMatroskaWriteStyle(*reinterpret_cast<QString*>(_v)); break;
        case 12: _t->setCustomGenres(*reinterpret_cast<QStringList*>(_v)); break;
        case 13: _t->setCustomFrames(*reinterpret_cast<QStringList*>(_v)); break;
        case 14: _t->setId3v2Version(*reinterpret_cast<int*>(_v)); break;
        case 15: _t->setTextEncodingV1(*reinterpret_cast<QString*>(_v)); break;
        case 16: _t->setTextEncodingV1Index(*reinterpret_cast<int*>(_v)); break;
        case 17: _t->setTextEncoding(*reinterpret_cast<int*>(_v)); break;
        case 18: _t->setQuickAccessFrames(*reinterpret_cast<quint64*>(_v)); break;
        case 19: _t->setQuickAccessFrameOrder(*reinterpret_cast<QList<int>*>(_v)); break;
        case 20: _t->setTrackNumberDigits(*reinterpret_cast<int*>(_v)); break;
        case 21: _t->setOnlyCustomGenres(*reinterpret_cast<bool*>(_v)); break;
        case 22: _t->setPluginOrder(*reinterpret_cast<QStringList*>(_v)); break;
        case 23: _t->setDisabledPlugins(*reinterpret_cast<QStringList*>(_v)); break;
        case 24: _t->setAvailablePlugins(*reinterpret_cast<QStringList*>(_v)); break;
        case 25: _t->setStarRatingMappingStrings(*reinterpret_cast<QStringList*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *TagConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TagConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9TagConfigE_t>.strings))
        return static_cast<void*>(this);
    return StoredConfig<TagConfig>::qt_metacast(_clname);
}

int TagConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StoredConfig<TagConfig>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 37)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 37;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 37)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 37;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    }
    return _id;
}

// SIGNAL 0
void TagConfig::taggedFileFeaturesChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void TagConfig::markTruncationsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void TagConfig::maximumPictureSizeChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void TagConfig::markOversizedPicturesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void TagConfig::markStandardViolationsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void TagConfig::enableTotalNumberOfTracksChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void TagConfig::genreNotNumericChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void TagConfig::lowercaseId3RiffChunkChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void TagConfig::commentNameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void TagConfig::pictureNameIndexChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void TagConfig::riffTrackNameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void TagConfig::matroskaWriteStyleChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void TagConfig::customGenresChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void TagConfig::customFramesChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void TagConfig::id3v2VersionChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void TagConfig::textEncodingV1Changed(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void TagConfig::textEncodingChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void TagConfig::quickAccessFramesChanged(quint64 _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1);
}

// SIGNAL 18
void TagConfig::quickAccessFrameOrderChanged(const QList<int> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1);
}

// SIGNAL 19
void TagConfig::trackNumberDigitsChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}

// SIGNAL 20
void TagConfig::onlyCustomGenresChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 20, nullptr, _t1);
}

// SIGNAL 21
void TagConfig::pluginOrderChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 21, nullptr, _t1);
}

// SIGNAL 22
void TagConfig::disabledPluginsChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 22, nullptr, _t1);
}

// SIGNAL 23
void TagConfig::availablePluginsChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 23, nullptr, _t1);
}

// SIGNAL 24
void TagConfig::starRatingMappingsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 24, nullptr);
}
QT_WARNING_POP
