/****************************************************************************
** Meta object code from reading C++ file 'fileconfig.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/core/config/fileconfig.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileconfig.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10FileConfigE_t {};
} // unnamed namespace

template <> constexpr inline auto FileConfig::qt_create_metaobjectdata<qt_meta_tag_ZN10FileConfigE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FileConfig",
        "nameFilterChanged",
        "",
        "nameFilter",
        "includeFoldersChanged",
        "includeFolders",
        "excludeFoldersChanged",
        "excludeFolders",
        "showHiddenFilesChanged",
        "showHiddenFiles",
        "sortIgnoringPunctuationChanged",
        "sortIgnoringPunctuation",
        "toFilenameFormatChanged",
        "toFilenameFormat",
        "toFilenameFormatsChanged",
        "toFilenameFormats",
        "fromFilenameFormatChanged",
        "fromFilenameFormat",
        "fromFilenameFormatsChanged",
        "fromFilenameFormats",
        "defaultCoverFileNameChanged",
        "defaultCoverFileName",
        "lastOpenedFileChanged",
        "lastOpenedFile",
        "textEncodingChanged",
        "textEncoding",
        "preserveTimeChanged",
        "preserveTime",
        "markChangesChanged",
        "markChanges",
        "loadLastOpenedFileChanged",
        "loadLastOpenedFile",
        "textEncodingIndex"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'nameFilterChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'includeFoldersChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 5 },
        }}),
        // Signal 'excludeFoldersChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 7 },
        }}),
        // Signal 'showHiddenFilesChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'sortIgnoringPunctuationChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Signal 'toFilenameFormatChanged'
        QtMocHelpers::SignalData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'toFilenameFormatsChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 15 },
        }}),
        // Signal 'fromFilenameFormatChanged'
        QtMocHelpers::SignalData<void(const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Signal 'fromFilenameFormatsChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 19 },
        }}),
        // Signal 'defaultCoverFileNameChanged'
        QtMocHelpers::SignalData<void(const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 21 },
        }}),
        // Signal 'lastOpenedFileChanged'
        QtMocHelpers::SignalData<void(const QString &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Signal 'textEncodingChanged'
        QtMocHelpers::SignalData<void(const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 25 },
        }}),
        // Signal 'preserveTimeChanged'
        QtMocHelpers::SignalData<void(bool)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 27 },
        }}),
        // Signal 'markChangesChanged'
        QtMocHelpers::SignalData<void(bool)>(28, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 29 },
        }}),
        // Signal 'loadLastOpenedFileChanged'
        QtMocHelpers::SignalData<void(bool)>(30, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 31 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'nameFilter'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'includeFolders'
        QtMocHelpers::PropertyData<QStringList>(5, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'excludeFolders'
        QtMocHelpers::PropertyData<QStringList>(7, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'showHiddenFiles'
        QtMocHelpers::PropertyData<bool>(9, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'sortIgnoringPunctuation'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'toFilenameFormat'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'toFilenameFormats'
        QtMocHelpers::PropertyData<QStringList>(15, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'fromFilenameFormat'
        QtMocHelpers::PropertyData<QString>(17, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'fromFilenameFormats'
        QtMocHelpers::PropertyData<QStringList>(19, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'defaultCoverFileName'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'lastOpenedFile'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'textEncoding'
        QtMocHelpers::PropertyData<QString>(25, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'textEncodingIndex'
        QtMocHelpers::PropertyData<int>(32, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'preserveTime'
        QtMocHelpers::PropertyData<bool>(27, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 12),
        // property 'markChanges'
        QtMocHelpers::PropertyData<bool>(29, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'loadLastOpenedFile'
        QtMocHelpers::PropertyData<bool>(31, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 14),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FileConfig, qt_meta_tag_ZN10FileConfigE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FileConfig::staticMetaObject = { {
    QMetaObject::SuperData::link<StoredConfig<FileConfig>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10FileConfigE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10FileConfigE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10FileConfigE_t>.metaTypes,
    nullptr
} };

void FileConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FileConfig *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->nameFilterChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->includeFoldersChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 2: _t->excludeFoldersChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 3: _t->showHiddenFilesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->sortIgnoringPunctuationChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->toFilenameFormatChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->toFilenameFormatsChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 7: _t->fromFilenameFormatChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 8: _t->fromFilenameFormatsChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 9: _t->defaultCoverFileNameChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 10: _t->lastOpenedFileChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->textEncodingChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 12: _t->preserveTimeChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->markChangesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->loadLastOpenedFileChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FileConfig::*)(const QString & )>(_a, &FileConfig::nameFilterChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileConfig::*)(const QStringList & )>(_a, &FileConfig::includeFoldersChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileConfig::*)(const QStringList & )>(_a, &FileConfig::excludeFoldersChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileConfig::*)(bool )>(_a, &FileConfig::showHiddenFilesChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileConfig::*)(bool )>(_a, &FileConfig::sortIgnoringPunctuationChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileConfig::*)(const QString & )>(_a, &FileConfig::toFilenameFormatChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileConfig::*)(const QStringList & )>(_a, &FileConfig::toFilenameFormatsChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileConfig::*)(const QString & )>(_a, &FileConfig::fromFilenameFormatChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileConfig::*)(const QStringList & )>(_a, &FileConfig::fromFilenameFormatsChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileConfig::*)(const QString & )>(_a, &FileConfig::defaultCoverFileNameChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileConfig::*)(const QString & )>(_a, &FileConfig::lastOpenedFileChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileConfig::*)(const QString & )>(_a, &FileConfig::textEncodingChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileConfig::*)(bool )>(_a, &FileConfig::preserveTimeChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileConfig::*)(bool )>(_a, &FileConfig::markChangesChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileConfig::*)(bool )>(_a, &FileConfig::loadLastOpenedFileChanged, 14))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->nameFilter(); break;
        case 1: *reinterpret_cast<QStringList*>(_v) = _t->includeFolders(); break;
        case 2: *reinterpret_cast<QStringList*>(_v) = _t->excludeFolders(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->showHiddenFiles(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->sortIgnoringPunctuation(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->toFilenameFormat(); break;
        case 6: *reinterpret_cast<QStringList*>(_v) = _t->toFilenameFormats(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->fromFilenameFormat(); break;
        case 8: *reinterpret_cast<QStringList*>(_v) = _t->fromFilenameFormats(); break;
        case 9: *reinterpret_cast<QString*>(_v) = _t->defaultCoverFileName(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->lastOpenedFile(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->textEncoding(); break;
        case 12: *reinterpret_cast<int*>(_v) = _t->textEncodingIndex(); break;
        case 13: *reinterpret_cast<bool*>(_v) = _t->preserveTime(); break;
        case 14: *reinterpret_cast<bool*>(_v) = _t->markChanges(); break;
        case 15: *reinterpret_cast<bool*>(_v) = _t->loadLastOpenedFile(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setNameFilter(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setIncludeFolders(*reinterpret_cast<QStringList*>(_v)); break;
        case 2: _t->setExcludeFolders(*reinterpret_cast<QStringList*>(_v)); break;
        case 3: _t->setShowHiddenFiles(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setSortIgnoringPunctuation(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setToFilenameFormat(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setToFilenameFormats(*reinterpret_cast<QStringList*>(_v)); break;
        case 7: _t->setFromFilenameFormat(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setFromFilenameFormats(*reinterpret_cast<QStringList*>(_v)); break;
        case 9: _t->setDefaultCoverFileName(*reinterpret_cast<QString*>(_v)); break;
        case 10: _t->setLastOpenedFile(*reinterpret_cast<QString*>(_v)); break;
        case 11: _t->setTextEncoding(*reinterpret_cast<QString*>(_v)); break;
        case 12: _t->setTextEncodingIndex(*reinterpret_cast<int*>(_v)); break;
        case 13: _t->setPreserveTime(*reinterpret_cast<bool*>(_v)); break;
        case 14: _t->setMarkChanges(*reinterpret_cast<bool*>(_v)); break;
        case 15: _t->setLoadLastOpenedFile(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *FileConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FileConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10FileConfigE_t>.strings))
        return static_cast<void*>(this);
    return StoredConfig<FileConfig>::qt_metacast(_clname);
}

int FileConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StoredConfig<FileConfig>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    }
    return _id;
}

// SIGNAL 0
void FileConfig::nameFilterChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void FileConfig::includeFoldersChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void FileConfig::excludeFoldersChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void FileConfig::showHiddenFilesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void FileConfig::sortIgnoringPunctuationChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void FileConfig::toFilenameFormatChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void FileConfig::toFilenameFormatsChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void FileConfig::fromFilenameFormatChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void FileConfig::fromFilenameFormatsChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void FileConfig::defaultCoverFileNameChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void FileConfig::lastOpenedFileChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void FileConfig::textEncodingChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void FileConfig::preserveTimeChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void FileConfig::markChangesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void FileConfig::loadLastOpenedFileChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}
QT_WARNING_POP
