/****************************************************************************
** Meta object code from reading C++ file 'findreplacedialog.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/gui/dialogs/findreplacedialog.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'findreplacedialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17FindReplaceDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto FindReplaceDialog::qt_create_metaobjectdata<qt_meta_tag_ZN17FindReplaceDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FindReplaceDialog",
        "findRequested",
        "",
        "TagSearcher::Parameters",
        "params",
        "replaceRequested",
        "replaceAllRequested",
        "showProgress",
        "msg",
        "find",
        "replace",
        "replaceAll",
        "saveConfig",
        "showHelp",
        "onReturnPressedInFind",
        "onReturnPressedInReplace"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'findRequested'
        QtMocHelpers::SignalData<void(const TagSearcher::Parameters &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'replaceRequested'
        QtMocHelpers::SignalData<void(const TagSearcher::Parameters &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'replaceAllRequested'
        QtMocHelpers::SignalData<void(const TagSearcher::Parameters &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Slot 'showProgress'
        QtMocHelpers::SlotData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Slot 'find'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'replace'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'replaceAll'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'saveConfig'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showHelp'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onReturnPressedInFind'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onReturnPressedInReplace'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FindReplaceDialog, qt_meta_tag_ZN17FindReplaceDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FindReplaceDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17FindReplaceDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17FindReplaceDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17FindReplaceDialogE_t>.metaTypes,
    nullptr
} };

void FindReplaceDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FindReplaceDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->findRequested((*reinterpret_cast<std::add_pointer_t<TagSearcher::Parameters>>(_a[1]))); break;
        case 1: _t->replaceRequested((*reinterpret_cast<std::add_pointer_t<TagSearcher::Parameters>>(_a[1]))); break;
        case 2: _t->replaceAllRequested((*reinterpret_cast<std::add_pointer_t<TagSearcher::Parameters>>(_a[1]))); break;
        case 3: _t->showProgress((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->find(); break;
        case 5: _t->replace(); break;
        case 6: _t->replaceAll(); break;
        case 7: _t->saveConfig(); break;
        case 8: _t->showHelp(); break;
        case 9: _t->onReturnPressedInFind(); break;
        case 10: _t->onReturnPressedInReplace(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FindReplaceDialog::*)(const TagSearcher::Parameters & )>(_a, &FindReplaceDialog::findRequested, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FindReplaceDialog::*)(const TagSearcher::Parameters & )>(_a, &FindReplaceDialog::replaceRequested, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FindReplaceDialog::*)(const TagSearcher::Parameters & )>(_a, &FindReplaceDialog::replaceAllRequested, 2))
            return;
    }
}

const QMetaObject *FindReplaceDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FindReplaceDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17FindReplaceDialogE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int FindReplaceDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void FindReplaceDialog::findRequested(const TagSearcher::Parameters & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void FindReplaceDialog::replaceRequested(const TagSearcher::Parameters & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void FindReplaceDialog::replaceAllRequested(const TagSearcher::Parameters & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
