/****************************************************************************
** Meta object code from reading C++ file 'batchimporter.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/core/import/batchimporter.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'batchimporter.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN13BatchImporterE_t {};
} // unnamed namespace

template <> constexpr inline auto BatchImporter::qt_create_metaobjectdata<qt_meta_tag_ZN13BatchImporterE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "BatchImporter",
        "reportImportEvent",
        "",
        "type",
        "text",
        "finished",
        "abort",
        "onFindFinished",
        "searchStr",
        "onFindProgress",
        "step",
        "total",
        "onAlbumFinished",
        "albumStr",
        "onAlbumProgress",
        "onImageDownloaded",
        "data",
        "mimeType",
        "url",
        "ImportEventType",
        "ReadingDirectory",
        "Started",
        "SourceSelected",
        "QueryingAlbumList",
        "FetchingTrackList",
        "TrackListReceived",
        "FetchingCoverArt",
        "CoverArtReceived",
        "Finished",
        "Aborted",
        "Error"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'reportImportEvent'
        QtMocHelpers::SignalData<void(int, const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::QString, 4 },
        }}),
        // Signal 'finished'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'abort'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onFindFinished'
        QtMocHelpers::SlotData<void(const QByteArray &)>(7, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 8 },
        }}),
        // Slot 'onFindProgress'
        QtMocHelpers::SlotData<void(const QString &, int, int)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 4 }, { QMetaType::Int, 10 }, { QMetaType::Int, 11 },
        }}),
        // Slot 'onAlbumFinished'
        QtMocHelpers::SlotData<void(const QByteArray &)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 13 },
        }}),
        // Slot 'onAlbumProgress'
        QtMocHelpers::SlotData<void(const QString &, int, int)>(14, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 4 }, { QMetaType::Int, 10 }, { QMetaType::Int, 11 },
        }}),
        // Slot 'onImageDownloaded'
        QtMocHelpers::SlotData<void(const QByteArray &, const QString &, const QString &)>(15, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QByteArray, 16 }, { QMetaType::QString, 17 }, { QMetaType::QString, 18 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'ImportEventType'
        QtMocHelpers::EnumData<enum ImportEventType>(19, 19, QMC::EnumFlags{}).add({
            {   20, ImportEventType::ReadingDirectory },
            {   21, ImportEventType::Started },
            {   22, ImportEventType::SourceSelected },
            {   23, ImportEventType::QueryingAlbumList },
            {   24, ImportEventType::FetchingTrackList },
            {   25, ImportEventType::TrackListReceived },
            {   26, ImportEventType::FetchingCoverArt },
            {   27, ImportEventType::CoverArtReceived },
            {   28, ImportEventType::Finished },
            {   29, ImportEventType::Aborted },
            {   30, ImportEventType::Error },
        }),
    };
    return QtMocHelpers::metaObjectData<BatchImporter, qt_meta_tag_ZN13BatchImporterE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject BatchImporter::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13BatchImporterE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13BatchImporterE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13BatchImporterE_t>.metaTypes,
    nullptr
} };

void BatchImporter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BatchImporter *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->reportImportEvent((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 1: _t->finished(); break;
        case 2: _t->abort(); break;
        case 3: _t->onFindFinished((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 4: _t->onFindProgress((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 5: _t->onAlbumFinished((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 6: _t->onAlbumProgress((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 7: _t->onImageDownloaded((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (BatchImporter::*)(int , const QString & )>(_a, &BatchImporter::reportImportEvent, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (BatchImporter::*)()>(_a, &BatchImporter::finished, 1))
            return;
    }
}

const QMetaObject *BatchImporter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BatchImporter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13BatchImporterE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "IAbortable"))
        return static_cast< IAbortable*>(this);
    return QObject::qt_metacast(_clname);
}

int BatchImporter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void BatchImporter::reportImportEvent(int _t1, const QString & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void BatchImporter::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
