/****************************************************************************
** Meta object code from reading C++ file 'framenotice.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/core/tags/framenotice.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'framenotice.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11FrameNoticeE_t {};
} // unnamed namespace

template <> constexpr inline auto FrameNotice::qt_create_metaobjectdata<qt_meta_tag_ZN11FrameNoticeE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FrameNotice",
        "Warning",
        "None",
        "Truncated",
        "TooLarge",
        "Unique",
        "NlForbidden",
        "CrForbidden",
        "OwnerEmpty",
        "Numeric",
        "NrTotal",
        "DayMonth",
        "HourMinute",
        "Year",
        "YearSpace",
        "IsoDate",
        "MusicalKey",
        "LanguageCode",
        "IsrcCode",
        "StringList",
        "ExcessSpace",
        "NumWarnings"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Warning'
        QtMocHelpers::EnumData<enum Warning>(1, 1, QMC::EnumFlags{}).add({
            {    2, Warning::None },
            {    3, Warning::Truncated },
            {    4, Warning::TooLarge },
            {    5, Warning::Unique },
            {    6, Warning::NlForbidden },
            {    7, Warning::CrForbidden },
            {    8, Warning::OwnerEmpty },
            {    9, Warning::Numeric },
            {   10, Warning::NrTotal },
            {   11, Warning::DayMonth },
            {   12, Warning::HourMinute },
            {   13, Warning::Year },
            {   14, Warning::YearSpace },
            {   15, Warning::IsoDate },
            {   16, Warning::MusicalKey },
            {   17, Warning::LanguageCode },
            {   18, Warning::IsrcCode },
            {   19, Warning::StringList },
            {   20, Warning::ExcessSpace },
            {   21, Warning::NumWarnings },
        }),
    };
    return QtMocHelpers::metaObjectData<FrameNotice, qt_meta_tag_ZN11FrameNoticeE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FrameNotice::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11FrameNoticeE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11FrameNoticeE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11FrameNoticeE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
