/****************************************************************************
** Meta object code from reading C++ file 'exportconfig.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/core/config/exportconfig.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'exportconfig.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN12ExportConfigE_t {};
} // unnamed namespace

template <> constexpr inline auto ExportConfig::qt_create_metaobjectdata<qt_meta_tag_ZN12ExportConfigE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ExportConfig",
        "exportSourceChanged",
        "",
        "Frame::TagVersion",
        "exportSource",
        "exportFormatNamesChanged",
        "exportFormatNames",
        "exportFormatHeadersChanged",
        "exportFormatHeaders",
        "exportFormatTracksChanged",
        "exportFormatTracks",
        "exportFormatTrailersChanged",
        "exportFormatTrailers",
        "exportFormatIndexChanged",
        "exportFormatIndex",
        "exportWindowGeometryChanged",
        "exportWindowGeometry"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'exportSourceChanged'
        QtMocHelpers::SignalData<void(Frame::TagVersion)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'exportFormatNamesChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 6 },
        }}),
        // Signal 'exportFormatHeadersChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 8 },
        }}),
        // Signal 'exportFormatTracksChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 10 },
        }}),
        // Signal 'exportFormatTrailersChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 12 },
        }}),
        // Signal 'exportFormatIndexChanged'
        QtMocHelpers::SignalData<void(int)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 14 },
        }}),
        // Signal 'exportWindowGeometryChanged'
        QtMocHelpers::SignalData<void(const QByteArray &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 16 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'exportSource'
        QtMocHelpers::PropertyData<int>(4, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'exportFormatNames'
        QtMocHelpers::PropertyData<QStringList>(6, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'exportFormatHeaders'
        QtMocHelpers::PropertyData<QStringList>(8, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'exportFormatTracks'
        QtMocHelpers::PropertyData<QStringList>(10, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'exportFormatTrailers'
        QtMocHelpers::PropertyData<QStringList>(12, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'exportFormatIndex'
        QtMocHelpers::PropertyData<int>(14, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'exportWindowGeometry'
        QtMocHelpers::PropertyData<QByteArray>(16, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ExportConfig, qt_meta_tag_ZN12ExportConfigE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ExportConfig::staticMetaObject = { {
    QMetaObject::SuperData::link<StoredConfig<ExportConfig>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12ExportConfigE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12ExportConfigE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN12ExportConfigE_t>.metaTypes,
    nullptr
} };

void ExportConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ExportConfig *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->exportSourceChanged((*reinterpret_cast<std::add_pointer_t<Frame::TagVersion>>(_a[1]))); break;
        case 1: _t->exportFormatNamesChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 2: _t->exportFormatHeadersChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 3: _t->exportFormatTracksChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 4: _t->exportFormatTrailersChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 5: _t->exportFormatIndexChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->exportWindowGeometryChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ExportConfig::*)(Frame::TagVersion )>(_a, &ExportConfig::exportSourceChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ExportConfig::*)(const QStringList & )>(_a, &ExportConfig::exportFormatNamesChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ExportConfig::*)(const QStringList & )>(_a, &ExportConfig::exportFormatHeadersChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ExportConfig::*)(const QStringList & )>(_a, &ExportConfig::exportFormatTracksChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ExportConfig::*)(const QStringList & )>(_a, &ExportConfig::exportFormatTrailersChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ExportConfig::*)(int )>(_a, &ExportConfig::exportFormatIndexChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ExportConfig::*)(const QByteArray & )>(_a, &ExportConfig::exportWindowGeometryChanged, 6))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->exportSource(); break;
        case 1: *reinterpret_cast<QStringList*>(_v) = _t->exportFormatNames(); break;
        case 2: *reinterpret_cast<QStringList*>(_v) = _t->exportFormatHeaders(); break;
        case 3: *reinterpret_cast<QStringList*>(_v) = _t->exportFormatTracks(); break;
        case 4: *reinterpret_cast<QStringList*>(_v) = _t->exportFormatTrailers(); break;
        case 5: *reinterpret_cast<int*>(_v) = _t->exportFormatIndex(); break;
        case 6: *reinterpret_cast<QByteArray*>(_v) = _t->exportWindowGeometry(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setExportSourceInt(*reinterpret_cast<int*>(_v)); break;
        case 1: _t->setExportFormatNames(*reinterpret_cast<QStringList*>(_v)); break;
        case 2: _t->setExportFormatHeaders(*reinterpret_cast<QStringList*>(_v)); break;
        case 3: _t->setExportFormatTracks(*reinterpret_cast<QStringList*>(_v)); break;
        case 4: _t->setExportFormatTrailers(*reinterpret_cast<QStringList*>(_v)); break;
        case 5: _t->setExportFormatIndex(*reinterpret_cast<int*>(_v)); break;
        case 6: _t->setExportWindowGeometry(*reinterpret_cast<QByteArray*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *ExportConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ExportConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN12ExportConfigE_t>.strings))
        return static_cast<void*>(this);
    return StoredConfig<ExportConfig>::qt_metacast(_clname);
}

int ExportConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StoredConfig<ExportConfig>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void ExportConfig::exportSourceChanged(Frame::TagVersion _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void ExportConfig::exportFormatNamesChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void ExportConfig::exportFormatHeadersChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void ExportConfig::exportFormatTracksChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void ExportConfig::exportFormatTrailersChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void ExportConfig::exportFormatIndexChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void ExportConfig::exportWindowGeometryChanged(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}
QT_WARNING_POP
