/****************************************************************************
** Meta object code from reading C++ file 'frameeditorobject.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/core/model/frameeditorobject.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'frameeditorobject.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17FrameEditorObjectE_t {};
} // unnamed namespace

template <> constexpr inline auto FrameEditorObject::qt_create_metaobjectdata<qt_meta_tag_ZN17FrameEditorObjectE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FrameEditorObject",
        "frameEdited",
        "",
        "Frame::TagNumber",
        "tagNr",
        "const Frame*",
        "frame",
        "frameSelected",
        "frameSelectionRequested",
        "frameNames",
        "frameEditRequested",
        "FrameObjectModel*",
        "onFrameSelectionFinished",
        "displayName",
        "onFrameEditFinished"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'frameEdited'
        QtMocHelpers::SignalData<void(Frame::TagNumber, const Frame *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'frameSelected'
        QtMocHelpers::SignalData<void(Frame::TagNumber, const Frame *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'frameSelectionRequested'
        QtMocHelpers::SignalData<void(const QStringList &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 9 },
        }}),
        // Signal 'frameEditRequested'
        QtMocHelpers::SignalData<void(FrameObjectModel *)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 6 },
        }}),
        // Slot 'onFrameSelectionFinished'
        QtMocHelpers::SlotData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'onFrameEditFinished'
        QtMocHelpers::SlotData<void(FrameObjectModel *)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FrameEditorObject, qt_meta_tag_ZN17FrameEditorObjectE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FrameEditorObject::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17FrameEditorObjectE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17FrameEditorObjectE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17FrameEditorObjectE_t>.metaTypes,
    nullptr
} };

void FrameEditorObject::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FrameEditorObject *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->frameEdited((*reinterpret_cast<std::add_pointer_t<Frame::TagNumber>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<const Frame*>>(_a[2]))); break;
        case 1: _t->frameSelected((*reinterpret_cast<std::add_pointer_t<Frame::TagNumber>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<const Frame*>>(_a[2]))); break;
        case 2: _t->frameSelectionRequested((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 3: _t->frameEditRequested((*reinterpret_cast<std::add_pointer_t<FrameObjectModel*>>(_a[1]))); break;
        case 4: _t->onFrameSelectionFinished((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->onFrameEditFinished((*reinterpret_cast<std::add_pointer_t<FrameObjectModel*>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FrameEditorObject::*)(Frame::TagNumber , const Frame * )>(_a, &FrameEditorObject::frameEdited, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameEditorObject::*)(Frame::TagNumber , const Frame * )>(_a, &FrameEditorObject::frameSelected, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameEditorObject::*)(const QStringList & )>(_a, &FrameEditorObject::frameSelectionRequested, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (FrameEditorObject::*)(FrameObjectModel * )>(_a, &FrameEditorObject::frameEditRequested, 3))
            return;
    }
}

const QMetaObject *FrameEditorObject::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FrameEditorObject::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17FrameEditorObjectE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "IFrameEditor"))
        return static_cast< IFrameEditor*>(this);
    return QObject::qt_metacast(_clname);
}

int FrameEditorObject::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void FrameEditorObject::frameEdited(Frame::TagNumber _t1, const Frame * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void FrameEditorObject::frameSelected(Frame::TagNumber _t1, const Frame * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void FrameEditorObject::frameSelectionRequested(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void FrameEditorObject::frameEditRequested(FrameObjectModel * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
