/****************************************************************************
** Meta object code from reading C++ file 'kid3form.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/gui/forms/kid3form.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kid3form.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN8Kid3FormE_t {};
} // unnamed namespace

template <> constexpr inline auto Kid3Form::qt_create_metaobjectdata<qt_meta_tag_ZN8Kid3FormE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kid3Form",
        "nameLineEditChanged",
        "",
        "txt",
        "dirSelected",
        "QModelIndex",
        "index",
        "fileActivated",
        "openParentDirectory",
        "setFocusFilename",
        "setFocusTag",
        "Frame::TagNumber",
        "tagNr",
        "setFocusNextTag",
        "setFocusPreviousTag",
        "setFocusFileList",
        "setFocusDirList",
        "selectAllFiles",
        "deselectAllFiles",
        "nextFile",
        "select",
        "onlyTaggedFiles",
        "previousFile",
        "selectNextTaggedFile",
        "selectPreviousTaggedFile",
        "setFileRootIndex",
        "setDirRootIndex",
        "showHideFile",
        "onFormatEditTextChanged",
        "text",
        "onFormatFromFilenameEditTextChanged",
        "onFirstDirectoryOpened",
        "copyTagsActionData",
        "setToFilenameFormats",
        "setFromFilenameFormats",
        "onFrameSelectionModelRowChanged",
        "current",
        "previous"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'nameLineEditChanged'
        QtMocHelpers::SlotData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Slot 'dirSelected'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'fileActivated'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'openParentDirectory'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'setFocusFilename'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setFocusTag'
        QtMocHelpers::SlotData<void(Frame::TagNumber)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'setFocusNextTag'
        QtMocHelpers::SlotData<void(Frame::TagNumber)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'setFocusPreviousTag'
        QtMocHelpers::SlotData<void(Frame::TagNumber)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'setFocusFileList'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setFocusDirList'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectAllFiles'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'deselectAllFiles'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'nextFile'
        QtMocHelpers::SlotData<bool(bool, bool)>(19, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Bool, 20 }, { QMetaType::Bool, 21 },
        }}),
        // Slot 'nextFile'
        QtMocHelpers::SlotData<bool(bool)>(19, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::Bool, 20 },
        }}),
        // Slot 'nextFile'
        QtMocHelpers::SlotData<bool()>(19, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Slot 'previousFile'
        QtMocHelpers::SlotData<bool(bool, bool)>(22, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Bool, 20 }, { QMetaType::Bool, 21 },
        }}),
        // Slot 'previousFile'
        QtMocHelpers::SlotData<bool(bool)>(22, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::Bool, 20 },
        }}),
        // Slot 'previousFile'
        QtMocHelpers::SlotData<bool()>(22, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool),
        // Slot 'selectNextTaggedFile'
        QtMocHelpers::SlotData<bool()>(23, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'selectPreviousTaggedFile'
        QtMocHelpers::SlotData<bool()>(24, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'setFileRootIndex'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'setDirRootIndex'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'showHideFile'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onFormatEditTextChanged'
        QtMocHelpers::SlotData<void(const QString &)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 29 },
        }}),
        // Slot 'onFormatFromFilenameEditTextChanged'
        QtMocHelpers::SlotData<void(const QString &)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 29 },
        }}),
        // Slot 'onFirstDirectoryOpened'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'copyTagsActionData'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setToFilenameFormats'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'setFromFilenameFormats'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onFrameSelectionModelRowChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(35, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 5, 36 }, { 0x80000000 | 5, 37 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Kid3Form, qt_meta_tag_ZN8Kid3FormE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kid3Form::staticMetaObject = { {
    QMetaObject::SuperData::link<QSplitter::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8Kid3FormE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8Kid3FormE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN8Kid3FormE_t>.metaTypes,
    nullptr
} };

void Kid3Form::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Kid3Form *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->nameLineEditChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->dirSelected((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 2: _t->fileActivated((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 3: _t->openParentDirectory((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 4: _t->setFocusFilename(); break;
        case 5: _t->setFocusTag((*reinterpret_cast<std::add_pointer_t<Frame::TagNumber>>(_a[1]))); break;
        case 6: _t->setFocusNextTag((*reinterpret_cast<std::add_pointer_t<Frame::TagNumber>>(_a[1]))); break;
        case 7: _t->setFocusPreviousTag((*reinterpret_cast<std::add_pointer_t<Frame::TagNumber>>(_a[1]))); break;
        case 8: _t->setFocusFileList(); break;
        case 9: _t->setFocusDirList(); break;
        case 10: _t->selectAllFiles(); break;
        case 11: _t->deselectAllFiles(); break;
        case 12: { bool _r = _t->nextFile((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 13: { bool _r = _t->nextFile((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 14: { bool _r = _t->nextFile();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 15: { bool _r = _t->previousFile((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 16: { bool _r = _t->previousFile((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->previousFile();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 18: { bool _r = _t->selectNextTaggedFile();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 19: { bool _r = _t->selectPreviousTaggedFile();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: _t->setFileRootIndex((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 21: _t->setDirRootIndex((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 22: _t->showHideFile(); break;
        case 23: _t->onFormatEditTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 24: _t->onFormatFromFilenameEditTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 25: _t->onFirstDirectoryOpened(); break;
        case 26: _t->copyTagsActionData(); break;
        case 27: _t->setToFilenameFormats(); break;
        case 28: _t->setFromFilenameFormats(); break;
        case 29: _t->onFrameSelectionModelRowChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        default: ;
        }
    }
}

const QMetaObject *Kid3Form::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kid3Form::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN8Kid3FormE_t>.strings))
        return static_cast<void*>(this);
    return QSplitter::qt_metacast(_clname);
}

int Kid3Form::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSplitter::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 30)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 30;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 30)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 30;
    }
    return _id;
}
namespace {
struct qt_meta_tag_ZN18Kid3FormTagContextE_t {};
} // unnamed namespace

template <> constexpr inline auto Kid3FormTagContext::qt_create_metaobjectdata<qt_meta_tag_ZN18Kid3FormTagContextE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kid3FormTagContext",
        "setFocusTag",
        "",
        "showHideTag"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'setFocusTag'
        QtMocHelpers::SlotData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showHideTag'
        QtMocHelpers::SlotData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Kid3FormTagContext, qt_meta_tag_ZN18Kid3FormTagContextE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kid3FormTagContext::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18Kid3FormTagContextE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18Kid3FormTagContextE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18Kid3FormTagContextE_t>.metaTypes,
    nullptr
} };

void Kid3FormTagContext::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Kid3FormTagContext *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setFocusTag(); break;
        case 1: _t->showHideTag(); break;
        default: ;
        }
    }
    (void)_a;
}

const QMetaObject *Kid3FormTagContext::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kid3FormTagContext::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18Kid3FormTagContextE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Kid3FormTagContext::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 2;
    }
    return _id;
}
QT_WARNING_POP
