/****************************************************************************
** Meta object code from reading C++ file 'fileinfogatherer_p.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/core/model/fileinfogatherer_p.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'fileinfogatherer_p.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16FileInfoGathererE_t {};
} // unnamed namespace

template <> constexpr inline auto FileInfoGatherer::qt_create_metaobjectdata<qt_meta_tag_ZN16FileInfoGathererE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FileInfoGatherer",
        "updates",
        "",
        "directory",
        "QList<std::pair<QString,QFileInfo>>",
        "newListOfFiles",
        "listOfFiles",
        "nameResolved",
        "fileName",
        "resolvedName",
        "directoryLoaded",
        "path",
        "list",
        "directoryPath",
        "fetchExtendedInformation",
        "files",
        "updateFile",
        "filePath",
        "setResolveSymlinks",
        "enable",
        "setDecorationProvider",
        "AbstractFileDecorationProvider*",
        "provider",
        "driveAdded",
        "driveRemoved"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'updates'
        QtMocHelpers::SignalData<void(const QString &, const QVector<QPair<QString,QFileInfo> > &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { 0x80000000 | 4, 1 },
        }}),
        // Signal 'newListOfFiles'
        QtMocHelpers::SignalData<void(const QString &, const QStringList &) const>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 }, { QMetaType::QStringList, 6 },
        }}),
        // Signal 'nameResolved'
        QtMocHelpers::SignalData<void(const QString &, const QString &) const>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 }, { QMetaType::QString, 9 },
        }}),
        // Signal 'directoryLoaded'
        QtMocHelpers::SignalData<void(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Slot 'list'
        QtMocHelpers::SlotData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Slot 'fetchExtendedInformation'
        QtMocHelpers::SlotData<void(const QString &, const QStringList &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 }, { QMetaType::QStringList, 15 },
        }}),
        // Slot 'updateFile'
        QtMocHelpers::SlotData<void(const QString &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 17 },
        }}),
        // Slot 'setResolveSymlinks'
        QtMocHelpers::SlotData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Slot 'setDecorationProvider'
        QtMocHelpers::SlotData<void(AbstractFileDecorationProvider *)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 21, 22 },
        }}),
        // Slot 'driveAdded'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'driveRemoved'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FileInfoGatherer, qt_meta_tag_ZN16FileInfoGathererE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FileInfoGatherer::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16FileInfoGathererE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16FileInfoGathererE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16FileInfoGathererE_t>.metaTypes,
    nullptr
} };

void FileInfoGatherer::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FileInfoGatherer *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->updates((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QList<std::pair<QString,QFileInfo>>>>(_a[2]))); break;
        case 1: _t->newListOfFiles((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 2: _t->nameResolved((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 3: _t->directoryLoaded((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->list((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->fetchExtendedInformation((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 6: _t->updateFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->setResolveSymlinks((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->setDecorationProvider((*reinterpret_cast<std::add_pointer_t<AbstractFileDecorationProvider*>>(_a[1]))); break;
        case 9: _t->driveAdded(); break;
        case 10: _t->driveRemoved(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FileInfoGatherer::*)(const QString & , const QVector<QPair<QString,QFileInfo> > & )>(_a, &FileInfoGatherer::updates, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileInfoGatherer::*)(const QString & , const QStringList & ) const>(_a, &FileInfoGatherer::newListOfFiles, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileInfoGatherer::*)(const QString & , const QString & ) const>(_a, &FileInfoGatherer::nameResolved, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (FileInfoGatherer::*)(const QString & )>(_a, &FileInfoGatherer::directoryLoaded, 3))
            return;
    }
}

const QMetaObject *FileInfoGatherer::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FileInfoGatherer::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16FileInfoGathererE_t>.strings))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int FileInfoGatherer::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void FileInfoGatherer::updates(const QString & _t1, const QVector<QPair<QString,QFileInfo> > & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void FileInfoGatherer::newListOfFiles(const QString & _t1, const QStringList & _t2)const
{
    QMetaObject::activate<void>(const_cast< FileInfoGatherer *>(this), &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void FileInfoGatherer::nameResolved(const QString & _t1, const QString & _t2)const
{
    QMetaObject::activate<void>(const_cast< FileInfoGatherer *>(this), &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void FileInfoGatherer::directoryLoaded(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
