/****************************************************************************
** Meta object code from reading C++ file 'scriptutils.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/plugins/kid3qml/scriptutils.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'scriptutils.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11ScriptUtilsE_t {};
} // unnamed namespace

template <> constexpr inline auto ScriptUtils::qt_create_metaobjectdata<qt_meta_tag_ZN11ScriptUtilsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ScriptUtils",
        "toStringList",
        "",
        "QList<QUrl>",
        "urls",
        "toPersistentModelIndexList",
        "QList<QPersistentModelIndex>",
        "QVariantList",
        "lst",
        "toTagVersion",
        "Frame::TagVersion",
        "nr",
        "toTagNumber",
        "Frame::TagNumber",
        "getRoleData",
        "QVariant",
        "modelObj",
        "row",
        "roleName",
        "QModelIndex",
        "parent",
        "setRoleData",
        "value",
        "getIndexRoleData",
        "index",
        "properties",
        "const QObject*",
        "obj",
        "getFieldIdNames",
        "getTextEncodingNames",
        "getTimestampFormatNames",
        "getPictureTypeNames",
        "getContentTypeNames",
        "writeFile",
        "filePath",
        "data",
        "readFile",
        "removeFile",
        "fileExists",
        "fileIsWritable",
        "getFilePermissions",
        "setFilePermissions",
        "modeBits",
        "classifyFile",
        "renameFile",
        "oldName",
        "newName",
        "copyFile",
        "source",
        "dest",
        "makeDir",
        "path",
        "removeDir",
        "tempPath",
        "musicPath",
        "mountedVolumes",
        "listDir",
        "nameFilters",
        "classify",
        "system",
        "program",
        "args",
        "msecs",
        "systemAsync",
        "QJSValue",
        "callback",
        "getEnv",
        "varName",
        "setEnv",
        "getKid3Version",
        "getKid3ReleaseYear",
        "getQtVersion",
        "getDataMd5",
        "getDataSize",
        "dataToImage",
        "format",
        "dataFromImage",
        "var",
        "loadImage",
        "saveImage",
        "imageProperties",
        "QVariantMap",
        "scaleImage",
        "width",
        "height"
    };

    QtMocHelpers::UintData qt_methods {
        // Method 'toStringList'
        QtMocHelpers::MethodData<QStringList(const QList<QUrl> &)>(1, 2, QMC::AccessPublic, QMetaType::QStringList, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Method 'toPersistentModelIndexList'
        QtMocHelpers::MethodData<QList<QPersistentModelIndex>(const QVariantList &)>(5, 2, QMC::AccessPublic, 0x80000000 | 6, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Method 'toTagVersion'
        QtMocHelpers::MethodData<Frame::TagVersion(int)>(9, 2, QMC::AccessPublic, 0x80000000 | 10, {{
            { QMetaType::Int, 11 },
        }}),
        // Method 'toTagNumber'
        QtMocHelpers::MethodData<Frame::TagNumber(int)>(12, 2, QMC::AccessPublic, 0x80000000 | 13, {{
            { QMetaType::Int, 11 },
        }}),
        // Method 'getRoleData'
        QtMocHelpers::MethodData<QVariant(QObject *, int, const QByteArray &, const QModelIndex &)>(14, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { QMetaType::QObjectStar, 16 }, { QMetaType::Int, 17 }, { QMetaType::QByteArray, 18 }, { 0x80000000 | 19, 20 },
        }}),
        // Method 'getRoleData'
        QtMocHelpers::MethodData<QVariant(QObject *, int, const QByteArray &)>(14, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 15, {{
            { QMetaType::QObjectStar, 16 }, { QMetaType::Int, 17 }, { QMetaType::QByteArray, 18 },
        }}),
        // Method 'setRoleData'
        QtMocHelpers::MethodData<bool(QObject *, int, const QByteArray &, const QVariant &, const QModelIndex &)>(21, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QObjectStar, 16 }, { QMetaType::Int, 17 }, { QMetaType::QByteArray, 18 }, { 0x80000000 | 15, 22 },
            { 0x80000000 | 19, 20 },
        }}),
        // Method 'setRoleData'
        QtMocHelpers::MethodData<bool(QObject *, int, const QByteArray &, const QVariant &)>(21, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { QMetaType::QObjectStar, 16 }, { QMetaType::Int, 17 }, { QMetaType::QByteArray, 18 }, { 0x80000000 | 15, 22 },
        }}),
        // Method 'getIndexRoleData'
        QtMocHelpers::MethodData<QVariant(const QModelIndex &, const QByteArray &)>(23, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { 0x80000000 | 19, 24 }, { QMetaType::QByteArray, 18 },
        }}),
        // Method 'properties'
        QtMocHelpers::MethodData<QString(const QObject *)>(25, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 26, 27 },
        }}),
        // Method 'getFieldIdNames'
        QtMocHelpers::MethodData<QStringList()>(28, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'getTextEncodingNames'
        QtMocHelpers::MethodData<QStringList()>(29, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'getTimestampFormatNames'
        QtMocHelpers::MethodData<QStringList()>(30, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'getPictureTypeNames'
        QtMocHelpers::MethodData<QStringList()>(31, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'getContentTypeNames'
        QtMocHelpers::MethodData<QStringList()>(32, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'writeFile'
        QtMocHelpers::MethodData<bool(const QString &, const QByteArray &)>(33, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 34 }, { QMetaType::QByteArray, 35 },
        }}),
        // Method 'readFile'
        QtMocHelpers::MethodData<QByteArray(const QString &)>(36, 2, QMC::AccessPublic, QMetaType::QByteArray, {{
            { QMetaType::QString, 34 },
        }}),
        // Method 'removeFile'
        QtMocHelpers::MethodData<bool(const QString &)>(37, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 34 },
        }}),
        // Method 'fileExists'
        QtMocHelpers::MethodData<bool(const QString &)>(38, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 34 },
        }}),
        // Method 'fileIsWritable'
        QtMocHelpers::MethodData<bool(const QString &)>(39, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 34 },
        }}),
        // Method 'getFilePermissions'
        QtMocHelpers::MethodData<int(const QString &)>(40, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QString, 34 },
        }}),
        // Method 'setFilePermissions'
        QtMocHelpers::MethodData<bool(const QString &, int)>(41, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 34 }, { QMetaType::Int, 42 },
        }}),
        // Method 'classifyFile'
        QtMocHelpers::MethodData<QString(const QString &)>(43, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QString, 34 },
        }}),
        // Method 'renameFile'
        QtMocHelpers::MethodData<bool(const QString &, const QString &)>(44, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 45 }, { QMetaType::QString, 46 },
        }}),
        // Method 'copyFile'
        QtMocHelpers::MethodData<bool(const QString &, const QString &)>(47, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 48 }, { QMetaType::QString, 49 },
        }}),
        // Method 'makeDir'
        QtMocHelpers::MethodData<bool(const QString &)>(50, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 51 },
        }}),
        // Method 'removeDir'
        QtMocHelpers::MethodData<bool(const QString &)>(52, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 51 },
        }}),
        // Method 'tempPath'
        QtMocHelpers::MethodData<QString()>(53, 2, QMC::AccessPublic, QMetaType::QString),
        // Method 'musicPath'
        QtMocHelpers::MethodData<QString()>(54, 2, QMC::AccessPublic, QMetaType::QString),
        // Method 'mountedVolumes'
        QtMocHelpers::MethodData<QVariantList()>(55, 2, QMC::AccessPublic, 0x80000000 | 7),
        // Method 'listDir'
        QtMocHelpers::MethodData<QStringList(const QString &, const QStringList &, bool)>(56, 2, QMC::AccessPublic, QMetaType::QStringList, {{
            { QMetaType::QString, 51 }, { QMetaType::QStringList, 57 }, { QMetaType::Bool, 58 },
        }}),
        // Method 'listDir'
        QtMocHelpers::MethodData<QStringList(const QString &, const QStringList &)>(56, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QStringList, {{
            { QMetaType::QString, 51 }, { QMetaType::QStringList, 57 },
        }}),
        // Method 'listDir'
        QtMocHelpers::MethodData<QStringList(const QString &)>(56, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QStringList, {{
            { QMetaType::QString, 51 },
        }}),
        // Method 'system'
        QtMocHelpers::MethodData<QVariantList(const QString &, const QStringList &, int)>(59, 2, QMC::AccessPublic, 0x80000000 | 7, {{
            { QMetaType::QString, 60 }, { QMetaType::QStringList, 61 }, { QMetaType::Int, 62 },
        }}),
        // Method 'system'
        QtMocHelpers::MethodData<QVariantList(const QString &, const QStringList &)>(59, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 7, {{
            { QMetaType::QString, 60 }, { QMetaType::QStringList, 61 },
        }}),
        // Method 'system'
        QtMocHelpers::MethodData<QVariantList(const QString &)>(59, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 7, {{
            { QMetaType::QString, 60 },
        }}),
        // Method 'systemAsync'
        QtMocHelpers::MethodData<void(const QString &, const QStringList &, QJSValue)>(63, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 60 }, { QMetaType::QStringList, 61 }, { 0x80000000 | 64, 65 },
        }}),
        // Method 'systemAsync'
        QtMocHelpers::MethodData<void(const QString &, const QStringList &)>(63, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 60 }, { QMetaType::QStringList, 61 },
        }}),
        // Method 'systemAsync'
        QtMocHelpers::MethodData<void(const QString &)>(63, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QString, 60 },
        }}),
        // Method 'getEnv'
        QtMocHelpers::MethodData<QByteArray(const QByteArray &)>(66, 2, QMC::AccessPublic, QMetaType::QByteArray, {{
            { QMetaType::QByteArray, 67 },
        }}),
        // Method 'setEnv'
        QtMocHelpers::MethodData<bool(const QByteArray &, const QByteArray &)>(68, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QByteArray, 67 }, { QMetaType::QByteArray, 22 },
        }}),
        // Method 'getKid3Version'
        QtMocHelpers::MethodData<QString()>(69, 2, QMC::AccessPublic, QMetaType::QString),
        // Method 'getKid3ReleaseYear'
        QtMocHelpers::MethodData<QString()>(70, 2, QMC::AccessPublic, QMetaType::QString),
        // Method 'getQtVersion'
        QtMocHelpers::MethodData<QString()>(71, 2, QMC::AccessPublic, QMetaType::QString),
        // Method 'getDataMd5'
        QtMocHelpers::MethodData<QString(const QByteArray &)>(72, 2, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::QByteArray, 35 },
        }}),
        // Method 'getDataSize'
        QtMocHelpers::MethodData<int(const QByteArray &)>(73, 2, QMC::AccessPublic, QMetaType::Int, {{
            { QMetaType::QByteArray, 35 },
        }}),
        // Method 'dataToImage'
        QtMocHelpers::MethodData<QVariant(const QByteArray &, const QByteArray &)>(74, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { QMetaType::QByteArray, 35 }, { QMetaType::QByteArray, 75 },
        }}),
        // Method 'dataToImage'
        QtMocHelpers::MethodData<QVariant(const QByteArray &)>(74, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 15, {{
            { QMetaType::QByteArray, 35 },
        }}),
        // Method 'dataFromImage'
        QtMocHelpers::MethodData<QByteArray(const QVariant &, const QByteArray &)>(76, 2, QMC::AccessPublic, QMetaType::QByteArray, {{
            { 0x80000000 | 15, 77 }, { QMetaType::QByteArray, 75 },
        }}),
        // Method 'dataFromImage'
        QtMocHelpers::MethodData<QByteArray(const QVariant &)>(76, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::QByteArray, {{
            { 0x80000000 | 15, 77 },
        }}),
        // Method 'loadImage'
        QtMocHelpers::MethodData<QVariant(const QString &)>(78, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { QMetaType::QString, 34 },
        }}),
        // Method 'saveImage'
        QtMocHelpers::MethodData<bool(const QVariant &, const QString &, const QByteArray &)>(79, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { 0x80000000 | 15, 77 }, { QMetaType::QString, 34 }, { QMetaType::QByteArray, 75 },
        }}),
        // Method 'saveImage'
        QtMocHelpers::MethodData<bool(const QVariant &, const QString &)>(79, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Bool, {{
            { 0x80000000 | 15, 77 }, { QMetaType::QString, 34 },
        }}),
        // Method 'imageProperties'
        QtMocHelpers::MethodData<QVariantMap(const QVariant &)>(80, 2, QMC::AccessPublic, 0x80000000 | 81, {{
            { 0x80000000 | 15, 77 },
        }}),
        // Method 'scaleImage'
        QtMocHelpers::MethodData<QVariant(const QVariant &, int, int)>(82, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { 0x80000000 | 15, 77 }, { QMetaType::Int, 83 }, { QMetaType::Int, 84 },
        }}),
        // Method 'scaleImage'
        QtMocHelpers::MethodData<QVariant(const QVariant &, int)>(82, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 15, {{
            { 0x80000000 | 15, 77 }, { QMetaType::Int, 83 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ScriptUtils, qt_meta_tag_ZN11ScriptUtilsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ScriptUtils::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11ScriptUtilsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11ScriptUtilsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11ScriptUtilsE_t>.metaTypes,
    nullptr
} };

void ScriptUtils::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ScriptUtils *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { QStringList _r = _t->toStringList((*reinterpret_cast<std::add_pointer_t<QList<QUrl>>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 1: { QList<QPersistentModelIndex> _r = _t->toPersistentModelIndexList((*reinterpret_cast<std::add_pointer_t<QVariantList>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QList<QPersistentModelIndex>*>(_a[0]) = std::move(_r); }  break;
        case 2: { Frame::TagVersion _r = _t->toTagVersion((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<Frame::TagVersion*>(_a[0]) = std::move(_r); }  break;
        case 3: { Frame::TagNumber _r = _t->toTagNumber((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<Frame::TagNumber*>(_a[0]) = std::move(_r); }  break;
        case 4: { QVariant _r = _t->getRoleData((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[4])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 5: { QVariant _r = _t->getRoleData((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 6: { bool _r = _t->setRoleData((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[5])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 7: { bool _r = _t->setRoleData((*reinterpret_cast<std::add_pointer_t<QObject*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[4])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 8: { QVariant _r = _t->getIndexRoleData((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 9: { QString _r = _t->properties((*reinterpret_cast<std::add_pointer_t<const QObject*>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 10: { QStringList _r = _t->getFieldIdNames();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 11: { QStringList _r = _t->getTextEncodingNames();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 12: { QStringList _r = _t->getTimestampFormatNames();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 13: { QStringList _r = _t->getPictureTypeNames();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 14: { QStringList _r = _t->getContentTypeNames();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 15: { bool _r = _t->writeFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 16: { QByteArray _r = _t->readFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QByteArray*>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->removeFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 18: { bool _r = _t->fileExists((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 19: { bool _r = _t->fileIsWritable((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { int _r = _t->getFilePermissions((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->setFilePermissions((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: { QString _r = _t->classifyFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 23: { bool _r = _t->renameFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: { bool _r = _t->copyFile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 25: { bool _r = _t->makeDir((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 26: { bool _r = _t->removeDir((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 27: { QString _r = _t->tempPath();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 28: { QString _r = _t->musicPath();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 29: { QVariantList _r = _t->mountedVolumes();
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 30: { QStringList _r = _t->listDir((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 31: { QStringList _r = _t->listDir((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 32: { QStringList _r = _t->listDir((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 33: { QVariantList _r = _t->system((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 34: { QVariantList _r = _t->system((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 35: { QVariantList _r = _t->system((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantList*>(_a[0]) = std::move(_r); }  break;
        case 36: _t->systemAsync((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QJSValue>>(_a[3]))); break;
        case 37: _t->systemAsync((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2]))); break;
        case 38: _t->systemAsync((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 39: { QByteArray _r = _t->getEnv((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QByteArray*>(_a[0]) = std::move(_r); }  break;
        case 40: { bool _r = _t->setEnv((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 41: { QString _r = _t->getKid3Version();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 42: { QString _r = _t->getKid3ReleaseYear();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 43: { QString _r = _t->getQtVersion();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 44: { QString _r = _t->getDataMd5((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 45: { int _r = _t->getDataSize((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<int*>(_a[0]) = std::move(_r); }  break;
        case 46: { QVariant _r = _t->dataToImage((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 47: { QVariant _r = _t->dataToImage((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 48: { QByteArray _r = _t->dataFromImage((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QByteArray*>(_a[0]) = std::move(_r); }  break;
        case 49: { QByteArray _r = _t->dataFromImage((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QByteArray*>(_a[0]) = std::move(_r); }  break;
        case 50: { QVariant _r = _t->loadImage((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 51: { bool _r = _t->saveImage((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 52: { bool _r = _t->saveImage((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 53: { QVariantMap _r = _t->imageProperties((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QVariantMap*>(_a[0]) = std::move(_r); }  break;
        case 54: { QVariant _r = _t->scaleImage((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 55: { QVariant _r = _t->scaleImage((*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 36:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QJSValue >(); break;
            }
            break;
        }
    }
}

const QMetaObject *ScriptUtils::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ScriptUtils::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11ScriptUtilsE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ScriptUtils::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 56)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 56;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 56)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 56;
    }
    return _id;
}
QT_WARNING_POP
