/****************************************************************************
** Meta object code from reading C++ file 'scriptinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/core/model/scriptinterface.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'scriptinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15ScriptInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto ScriptInterface::qt_create_metaobjectdata<qt_meta_tag_ZN15ScriptInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ScriptInterface",
        "D-Bus Interface",
        "org.kde.Kid3",
        "openDirectory",
        "",
        "path",
        "unloadAllTags",
        "save",
        "getErrorMessage",
        "revert",
        "importFromFile",
        "tagMask",
        "fmtIdx",
        "importFromTags",
        "source",
        "extraction",
        "importFromTagsToSelection",
        "batchImport",
        "profileName",
        "downloadAlbumArt",
        "url",
        "allFilesInDir",
        "exportToFile",
        "createPlaylist",
        "getPlaylistItems",
        "setPlaylistItems",
        "items",
        "quit",
        "selectAll",
        "deselectAll",
        "firstFile",
        "previousFile",
        "nextFile",
        "selectFirstFile",
        "selectPreviousFile",
        "selectNextFile",
        "selectCurrentFile",
        "expandDirectory",
        "expandFileList",
        "applyFilenameFormat",
        "applyTagFormat",
        "applyTextEncoding",
        "setDirNameFromTag",
        "format",
        "create",
        "numberTracks",
        "firstTrackNr",
        "filter",
        "expression",
        "convertToId3v24",
        "convertToId3v23",
        "getDirectoryName",
        "getFileName",
        "setFileName",
        "name",
        "setFileNameFormat",
        "setFileNameFromTag",
        "getFrame",
        "setFrame",
        "value",
        "getTag",
        "getInformation",
        "setTagFromFileName",
        "setTagFromOtherTag",
        "copyTag",
        "pasteTag",
        "removeTag",
        "reparseConfiguration",
        "playAudio",
        "onRenameActionsScheduled"
    };

    QtMocHelpers::UintData qt_methods {
        // Slot 'openDirectory'
        QtMocHelpers::SlotData<bool(const QString &)>(3, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'unloadAllTags'
        QtMocHelpers::SlotData<void()>(6, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'save'
        QtMocHelpers::SlotData<bool()>(7, 4, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'getErrorMessage'
        QtMocHelpers::SlotData<QString() const>(8, 4, QMC::AccessPublic, QMetaType::QString),
        // Slot 'revert'
        QtMocHelpers::SlotData<void()>(9, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'importFromFile'
        QtMocHelpers::SlotData<bool(int, const QString &, int)>(10, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 11 }, { QMetaType::QString, 5 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'importFromTags'
        QtMocHelpers::SlotData<void(int, const QString &, const QString &)>(13, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 }, { QMetaType::QString, 14 }, { QMetaType::QString, 15 },
        }}),
        // Slot 'importFromTagsToSelection'
        QtMocHelpers::SlotData<QStringList(int, const QString &, const QString &)>(16, 4, QMC::AccessPublic, QMetaType::QStringList, {{
            { QMetaType::Int, 11 }, { QMetaType::QString, 14 }, { QMetaType::QString, 15 },
        }}),
        // Slot 'batchImport'
        QtMocHelpers::SlotData<bool(int, const QString &)>(17, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 11 }, { QMetaType::QString, 18 },
        }}),
        // Slot 'downloadAlbumArt'
        QtMocHelpers::SlotData<void(const QString &, bool)>(19, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 20 }, { QMetaType::Bool, 21 },
        }}),
        // Slot 'exportToFile'
        QtMocHelpers::SlotData<bool(int, const QString &, int)>(22, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 11 }, { QMetaType::QString, 5 }, { QMetaType::Int, 12 },
        }}),
        // Slot 'createPlaylist'
        QtMocHelpers::SlotData<bool()>(23, 4, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'getPlaylistItems'
        QtMocHelpers::SlotData<QStringList(const QString &)>(24, 4, QMC::AccessPublic, QMetaType::QStringList, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'setPlaylistItems'
        QtMocHelpers::SlotData<bool(const QString &, const QStringList &)>(25, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::QString, 5 }, { QMetaType::QStringList, 26 },
        }}),
        // Slot 'quit'
        QtMocHelpers::SlotData<void()>(27, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'selectAll'
        QtMocHelpers::SlotData<void()>(28, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'deselectAll'
        QtMocHelpers::SlotData<void()>(29, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'firstFile'
        QtMocHelpers::SlotData<bool()>(30, 4, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'previousFile'
        QtMocHelpers::SlotData<bool()>(31, 4, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'nextFile'
        QtMocHelpers::SlotData<bool()>(32, 4, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'selectFirstFile'
        QtMocHelpers::SlotData<bool()>(33, 4, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'selectPreviousFile'
        QtMocHelpers::SlotData<bool()>(34, 4, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'selectNextFile'
        QtMocHelpers::SlotData<bool()>(35, 4, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'selectCurrentFile'
        QtMocHelpers::SlotData<bool()>(36, 4, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'expandDirectory'
        QtMocHelpers::SlotData<bool()>(37, 4, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'expandFileList'
        QtMocHelpers::SlotData<void()>(38, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'applyFilenameFormat'
        QtMocHelpers::SlotData<void()>(39, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'applyTagFormat'
        QtMocHelpers::SlotData<void()>(40, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'applyTextEncoding'
        QtMocHelpers::SlotData<void()>(41, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setDirNameFromTag'
        QtMocHelpers::SlotData<bool(int, const QString &, bool)>(42, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 11 }, { QMetaType::QString, 43 }, { QMetaType::Bool, 44 },
        }}),
        // Slot 'numberTracks'
        QtMocHelpers::SlotData<void(int, int)>(45, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 }, { QMetaType::Int, 46 },
        }}),
        // Slot 'filter'
        QtMocHelpers::SlotData<void(const QString &)>(47, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 48 },
        }}),
        // Slot 'convertToId3v24'
        QtMocHelpers::SlotData<void()>(49, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'convertToId3v23'
        QtMocHelpers::SlotData<void()>(50, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'getDirectoryName'
        QtMocHelpers::SlotData<QString()>(51, 4, QMC::AccessPublic, QMetaType::QString),
        // Slot 'getFileName'
        QtMocHelpers::SlotData<QString()>(52, 4, QMC::AccessPublic, QMetaType::QString),
        // Slot 'setFileName'
        QtMocHelpers::SlotData<void(const QString &)>(53, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 54 },
        }}),
        // Slot 'setFileNameFormat'
        QtMocHelpers::SlotData<void(const QString &)>(55, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 43 },
        }}),
        // Slot 'setFileNameFromTag'
        QtMocHelpers::SlotData<void(int)>(56, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'getFrame'
        QtMocHelpers::SlotData<QString(int, const QString &)>(57, 4, QMC::AccessPublic, QMetaType::QString, {{
            { QMetaType::Int, 11 }, { QMetaType::QString, 54 },
        }}),
        // Slot 'setFrame'
        QtMocHelpers::SlotData<bool(int, const QString &, const QString &)>(58, 4, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 11 }, { QMetaType::QString, 54 }, { QMetaType::QString, 59 },
        }}),
        // Slot 'getTag'
        QtMocHelpers::SlotData<QStringList(int)>(60, 4, QMC::AccessPublic, QMetaType::QStringList, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'getInformation'
        QtMocHelpers::SlotData<QStringList()>(61, 4, QMC::AccessPublic, QMetaType::QStringList),
        // Slot 'setTagFromFileName'
        QtMocHelpers::SlotData<void(int)>(62, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'setTagFromOtherTag'
        QtMocHelpers::SlotData<void(int)>(63, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'copyTag'
        QtMocHelpers::SlotData<void(int)>(64, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'pasteTag'
        QtMocHelpers::SlotData<void(int)>(65, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'removeTag'
        QtMocHelpers::SlotData<void(int)>(66, 4, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Slot 'reparseConfiguration'
        QtMocHelpers::SlotData<void()>(67, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'playAudio'
        QtMocHelpers::SlotData<void()>(68, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'onRenameActionsScheduled'
        QtMocHelpers::SlotData<void()>(69, 4, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<ScriptInterface, qt_meta_tag_ZN15ScriptInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject ScriptInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractAdaptor::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15ScriptInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15ScriptInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15ScriptInterfaceE_t>.metaTypes,
    nullptr
} };

void ScriptInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ScriptInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: { bool _r = _t->openDirectory((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 1: _t->unloadAllTags(); break;
        case 2: { bool _r = _t->save();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 3: { QString _r = _t->getErrorMessage();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 4: _t->revert(); break;
        case 5: { bool _r = _t->importFromFile((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 6: _t->importFromTags((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3]))); break;
        case 7: { QStringList _r = _t->importFromTagsToSelection((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])));
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 8: { bool _r = _t->batchImport((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 9: _t->downloadAlbumArt((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 10: { bool _r = _t->exportToFile((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 11: { bool _r = _t->createPlaylist();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 12: { QStringList _r = _t->getPlaylistItems((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 13: { bool _r = _t->setPlaylistItems((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[2])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->quit(); break;
        case 15: _t->selectAll(); break;
        case 16: _t->deselectAll(); break;
        case 17: { bool _r = _t->firstFile();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 18: { bool _r = _t->previousFile();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 19: { bool _r = _t->nextFile();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 20: { bool _r = _t->selectFirstFile();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 21: { bool _r = _t->selectPreviousFile();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 22: { bool _r = _t->selectNextFile();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 23: { bool _r = _t->selectCurrentFile();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 24: { bool _r = _t->expandDirectory();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 25: _t->expandFileList(); break;
        case 26: _t->applyFilenameFormat(); break;
        case 27: _t->applyTagFormat(); break;
        case 28: _t->applyTextEncoding(); break;
        case 29: { bool _r = _t->setDirNameFromTag((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 30: _t->numberTracks((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 31: _t->filter((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 32: _t->convertToId3v24(); break;
        case 33: _t->convertToId3v23(); break;
        case 34: { QString _r = _t->getDirectoryName();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 35: { QString _r = _t->getFileName();
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 36: _t->setFileName((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 37: _t->setFileNameFormat((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 38: _t->setFileNameFromTag((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 39: { QString _r = _t->getFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        case 40: { bool _r = _t->setFrame((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 41: { QStringList _r = _t->getTag((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 42: { QStringList _r = _t->getInformation();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 43: _t->setTagFromFileName((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 44: _t->setTagFromOtherTag((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 45: _t->copyTag((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 46: _t->pasteTag((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 47: _t->removeTag((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 48: _t->reparseConfiguration(); break;
        case 49: _t->playAudio(); break;
        case 50: _t->onRenameActionsScheduled(); break;
        default: ;
        }
    }
}

const QMetaObject *ScriptInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ScriptInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15ScriptInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractAdaptor::qt_metacast(_clname);
}

int ScriptInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractAdaptor::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 51)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 51;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 51)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 51;
    }
    return _id;
}
QT_WARNING_POP
