/****************************************************************************
** Meta object code from reading C++ file 'serverimporterconfig.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/core/config/serverimporterconfig.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'serverimporterconfig.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20ServerImporterConfigE_t {};
} // unnamed namespace

template <> constexpr inline auto ServerImporterConfig::qt_create_metaobjectdata<qt_meta_tag_ZN20ServerImporterConfigE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ServerImporterConfig",
        "serverChanged",
        "",
        "server",
        "cgiPathChanged",
        "cgiPath",
        "windowGeometryChanged",
        "windowGeometry",
        "cgiPathUsedChanged",
        "cgiPathUsed",
        "additionalTagsUsedChanged",
        "additionalTagsUsed",
        "standardTagsChanged",
        "standardTags",
        "additionalTagsChanged",
        "additionalTags",
        "coverArtChanged",
        "coverArt"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'serverChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'cgiPathChanged'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'windowGeometryChanged'
        QtMocHelpers::SignalData<void(const QByteArray &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 7 },
        }}),
        // Signal 'cgiPathUsedChanged'
        QtMocHelpers::SignalData<void(bool)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 9 },
        }}),
        // Signal 'additionalTagsUsedChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Signal 'standardTagsChanged'
        QtMocHelpers::SignalData<void(bool)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 13 },
        }}),
        // Signal 'additionalTagsChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Signal 'coverArtChanged'
        QtMocHelpers::SignalData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'server'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'cgiPath'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'windowGeometry'
        QtMocHelpers::PropertyData<QByteArray>(7, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'cgiPathUsed'
        QtMocHelpers::PropertyData<bool>(9, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'additionalTagsUsed'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'standardTags'
        QtMocHelpers::PropertyData<bool>(13, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'additionalTags'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'coverArt'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ServerImporterConfig, qt_meta_tag_ZN20ServerImporterConfigE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ServerImporterConfig::staticMetaObject = { {
    QMetaObject::SuperData::link<GeneralConfig::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20ServerImporterConfigE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20ServerImporterConfigE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20ServerImporterConfigE_t>.metaTypes,
    nullptr
} };

void ServerImporterConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ServerImporterConfig *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->serverChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->cgiPathChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->windowGeometryChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 3: _t->cgiPathUsedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->additionalTagsUsedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->standardTagsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 6: _t->additionalTagsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->coverArtChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ServerImporterConfig::*)(const QString & )>(_a, &ServerImporterConfig::serverChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ServerImporterConfig::*)(const QString & )>(_a, &ServerImporterConfig::cgiPathChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ServerImporterConfig::*)(const QByteArray & )>(_a, &ServerImporterConfig::windowGeometryChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ServerImporterConfig::*)(bool )>(_a, &ServerImporterConfig::cgiPathUsedChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ServerImporterConfig::*)(bool )>(_a, &ServerImporterConfig::additionalTagsUsedChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ServerImporterConfig::*)(bool )>(_a, &ServerImporterConfig::standardTagsChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ServerImporterConfig::*)(bool )>(_a, &ServerImporterConfig::additionalTagsChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ServerImporterConfig::*)(bool )>(_a, &ServerImporterConfig::coverArtChanged, 7))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->server(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->cgiPath(); break;
        case 2: *reinterpret_cast<QByteArray*>(_v) = _t->windowGeometry(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->cgiPathUsed(); break;
        case 4: *reinterpret_cast<bool*>(_v) = _t->additionalTagsUsed(); break;
        case 5: *reinterpret_cast<bool*>(_v) = _t->standardTags(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->additionalTags(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->coverArt(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setServer(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setCgiPath(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setWindowGeometry(*reinterpret_cast<QByteArray*>(_v)); break;
        case 3: _t->setCgiPathUsed(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setAdditionalTagsUsed(*reinterpret_cast<bool*>(_v)); break;
        case 5: _t->setStandardTags(*reinterpret_cast<bool*>(_v)); break;
        case 6: _t->setAdditionalTags(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setCoverArt(*reinterpret_cast<bool*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *ServerImporterConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ServerImporterConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20ServerImporterConfigE_t>.strings))
        return static_cast<void*>(this);
    return GeneralConfig::qt_metacast(_clname);
}

int ServerImporterConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = GeneralConfig::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 8)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 8)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 8;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    }
    return _id;
}

// SIGNAL 0
void ServerImporterConfig::serverChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void ServerImporterConfig::cgiPathChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void ServerImporterConfig::windowGeometryChanged(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void ServerImporterConfig::cgiPathUsedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void ServerImporterConfig::additionalTagsUsedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void ServerImporterConfig::standardTagsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void ServerImporterConfig::additionalTagsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void ServerImporterConfig::coverArtChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}
QT_WARNING_POP
