/****************************************************************************
** Meta object code from reading C++ file 'mainwindowconfig.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/core/config/mainwindowconfig.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindowconfig.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16MainWindowConfigE_t {};
} // unnamed namespace

template <> constexpr inline auto MainWindowConfig::qt_create_metaobjectdata<qt_meta_tag_ZN16MainWindowConfigE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "MainWindowConfig",
        "geometryChanged",
        "",
        "geometry",
        "windowStateChanged",
        "windowState",
        "languageChanged",
        "language",
        "fontFamilyChanged",
        "fontFamily",
        "fontSizeChanged",
        "fontSize",
        "styleChanged",
        "style",
        "useFontChanged",
        "useFont",
        "hideToolBarChanged",
        "hideToolBar",
        "hideStatusBarChanged",
        "hideStatusBar",
        "dontUseNativeDialogsChanged",
        "dontUseNativeDialogs",
        "qtQuickStyleChanged",
        "qtQuickStyle",
        "getQtQuickStyleNames",
        "availableLanguages"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'geometryChanged'
        QtMocHelpers::SignalData<void(const QByteArray &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 3 },
        }}),
        // Signal 'windowStateChanged'
        QtMocHelpers::SignalData<void(const QByteArray &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 5 },
        }}),
        // Signal 'languageChanged'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'fontFamilyChanged'
        QtMocHelpers::SignalData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'fontSizeChanged'
        QtMocHelpers::SignalData<void(int)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 11 },
        }}),
        // Signal 'styleChanged'
        QtMocHelpers::SignalData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'useFontChanged'
        QtMocHelpers::SignalData<void(bool)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 15 },
        }}),
        // Signal 'hideToolBarChanged'
        QtMocHelpers::SignalData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Signal 'hideStatusBarChanged'
        QtMocHelpers::SignalData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Signal 'dontUseNativeDialogsChanged'
        QtMocHelpers::SignalData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 21 },
        }}),
        // Signal 'qtQuickStyleChanged'
        QtMocHelpers::SignalData<void(const QString &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Method 'getQtQuickStyleNames'
        QtMocHelpers::MethodData<QStringList()>(24, 2, QMC::AccessPublic, QMetaType::QStringList),
        // Method 'availableLanguages'
        QtMocHelpers::MethodData<QStringList()>(25, 2, QMC::AccessPublic, QMetaType::QStringList),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'geometry'
        QtMocHelpers::PropertyData<QByteArray>(3, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'windowState'
        QtMocHelpers::PropertyData<QByteArray>(5, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'language'
        QtMocHelpers::PropertyData<QString>(7, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'fontFamily'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'fontSize'
        QtMocHelpers::PropertyData<int>(11, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'style'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'useFont'
        QtMocHelpers::PropertyData<bool>(15, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'hideToolBar'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'hideStatusBar'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'dontUseNativeDialogs'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'qtQuickStyle'
        QtMocHelpers::PropertyData<QString>(23, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<MainWindowConfig, qt_meta_tag_ZN16MainWindowConfigE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject MainWindowConfig::staticMetaObject = { {
    QMetaObject::SuperData::link<StoredConfig<MainWindowConfig>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16MainWindowConfigE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16MainWindowConfigE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16MainWindowConfigE_t>.metaTypes,
    nullptr
} };

void MainWindowConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<MainWindowConfig *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->geometryChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 1: _t->windowStateChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 2: _t->languageChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->fontFamilyChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->fontSizeChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->styleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->useFontChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->hideToolBarChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->hideStatusBarChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->dontUseNativeDialogsChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->qtQuickStyleChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: { QStringList _r = _t->getQtQuickStyleNames();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        case 12: { QStringList _r = _t->availableLanguages();
            if (_a[0]) *reinterpret_cast<QStringList*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (MainWindowConfig::*)(const QByteArray & )>(_a, &MainWindowConfig::geometryChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindowConfig::*)(const QByteArray & )>(_a, &MainWindowConfig::windowStateChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindowConfig::*)(const QString & )>(_a, &MainWindowConfig::languageChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindowConfig::*)(const QString & )>(_a, &MainWindowConfig::fontFamilyChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindowConfig::*)(int )>(_a, &MainWindowConfig::fontSizeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindowConfig::*)(const QString & )>(_a, &MainWindowConfig::styleChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindowConfig::*)(bool )>(_a, &MainWindowConfig::useFontChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindowConfig::*)(bool )>(_a, &MainWindowConfig::hideToolBarChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindowConfig::*)(bool )>(_a, &MainWindowConfig::hideStatusBarChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindowConfig::*)(bool )>(_a, &MainWindowConfig::dontUseNativeDialogsChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (MainWindowConfig::*)(const QString & )>(_a, &MainWindowConfig::qtQuickStyleChanged, 10))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QByteArray*>(_v) = _t->geometry(); break;
        case 1: *reinterpret_cast<QByteArray*>(_v) = _t->windowState(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->language(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->fontFamily(); break;
        case 4: *reinterpret_cast<int*>(_v) = _t->fontSize(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->style(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->useFont(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->hideToolBar(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->hideStatusBar(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->dontUseNativeDialogs(); break;
        case 10: *reinterpret_cast<QString*>(_v) = _t->qtQuickStyle(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setGeometry(*reinterpret_cast<QByteArray*>(_v)); break;
        case 1: _t->setWindowState(*reinterpret_cast<QByteArray*>(_v)); break;
        case 2: _t->setLanguage(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setFontFamily(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setFontSize(*reinterpret_cast<int*>(_v)); break;
        case 5: _t->setStyle(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setUseFont(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setHideToolBar(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setHideStatusBar(*reinterpret_cast<bool*>(_v)); break;
        case 9: _t->setDontUseNativeDialogs(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setQtQuickStyle(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *MainWindowConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindowConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16MainWindowConfigE_t>.strings))
        return static_cast<void*>(this);
    return StoredConfig<MainWindowConfig>::qt_metacast(_clname);
}

int MainWindowConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StoredConfig<MainWindowConfig>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 13;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void MainWindowConfig::geometryChanged(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void MainWindowConfig::windowStateChanged(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void MainWindowConfig::languageChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void MainWindowConfig::fontFamilyChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void MainWindowConfig::fontSizeChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void MainWindowConfig::styleChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void MainWindowConfig::useFontChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void MainWindowConfig::hideToolBarChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void MainWindowConfig::hideStatusBarChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void MainWindowConfig::dontUseNativeDialogsChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void MainWindowConfig::qtQuickStyleChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}
QT_WARNING_POP
