/****************************************************************************
** Meta object code from reading C++ file 'checkablelistmodel.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/plugins/kid3qml/checkablelistmodel.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'checkablelistmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18CheckableListModelE_t {};
} // unnamed namespace

template <> constexpr inline auto CheckableListModel::qt_create_metaobjectdata<qt_meta_tag_ZN18CheckableListModelE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "CheckableListModel",
        "sourceModelChanged",
        "",
        "selectionModelChanged",
        "rootIndexChanged",
        "currentRowChanged",
        "row",
        "onModelAboutToBeReset",
        "onModelReset",
        "onDataChanged",
        "QModelIndex",
        "topLeft",
        "bottomRight",
        "onRowsAboutToBeRemoved",
        "parent",
        "first",
        "last",
        "onRowsRemoved",
        "onRowsAboutToBeInserted",
        "onRowsInserted",
        "onSelectionChanged",
        "QItemSelection",
        "selected",
        "deselected",
        "onCurrentChanged",
        "current",
        "previous",
        "getDataValue",
        "QVariant",
        "roleName",
        "setDataValue",
        "value",
        "modelIndex",
        "parentModelIndex",
        "hasModelChildren",
        "sourceModel",
        "selectionModel",
        "rootIndex",
        "currentRow"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'sourceModelChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'selectionModelChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'rootIndexChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'currentRowChanged'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Slot 'onModelAboutToBeReset'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onModelReset'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onDataChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(9, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 10, 12 },
        }}),
        // Slot 'onRowsAboutToBeRemoved'
        QtMocHelpers::SlotData<void(const QModelIndex &, int, int)>(13, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 14 }, { QMetaType::Int, 15 }, { QMetaType::Int, 16 },
        }}),
        // Slot 'onRowsRemoved'
        QtMocHelpers::SlotData<void(const QModelIndex &, int, int)>(17, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 14 }, { QMetaType::Int, 15 }, { QMetaType::Int, 16 },
        }}),
        // Slot 'onRowsAboutToBeInserted'
        QtMocHelpers::SlotData<void(const QModelIndex &, int, int)>(18, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 14 }, { QMetaType::Int, 15 }, { QMetaType::Int, 16 },
        }}),
        // Slot 'onRowsInserted'
        QtMocHelpers::SlotData<void(const QModelIndex &, int, int)>(19, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 14 }, { QMetaType::Int, 15 }, { QMetaType::Int, 16 },
        }}),
        // Slot 'onSelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(20, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 21, 22 }, { 0x80000000 | 21, 23 },
        }}),
        // Slot 'onCurrentChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 10, 25 }, { 0x80000000 | 10, 26 },
        }}),
        // Method 'getDataValue'
        QtMocHelpers::MethodData<QVariant(int, const QByteArray &) const>(27, 2, QMC::AccessPublic, 0x80000000 | 28, {{
            { QMetaType::Int, 6 }, { QMetaType::QByteArray, 29 },
        }}),
        // Method 'setDataValue'
        QtMocHelpers::MethodData<bool(int, const QByteArray &, const QVariant &)>(30, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 6 }, { QMetaType::QByteArray, 29 }, { 0x80000000 | 28, 31 },
        }}),
        // Method 'modelIndex'
        QtMocHelpers::MethodData<QModelIndex(int) const>(32, 2, QMC::AccessPublic, 0x80000000 | 10, {{
            { QMetaType::Int, 6 },
        }}),
        // Method 'parentModelIndex'
        QtMocHelpers::MethodData<QModelIndex() const>(33, 2, QMC::AccessPublic, 0x80000000 | 10),
        // Method 'hasModelChildren'
        QtMocHelpers::MethodData<bool(int) const>(34, 2, QMC::AccessPublic, QMetaType::Bool, {{
            { QMetaType::Int, 6 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'sourceModel'
        QtMocHelpers::PropertyData<QObject*>(35, QMetaType::QObjectStar, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'selectionModel'
        QtMocHelpers::PropertyData<QObject*>(36, QMetaType::QObjectStar, QMC::DefaultPropertyFlags | QMC::Writable, 1),
        // property 'rootIndex'
        QtMocHelpers::PropertyData<QModelIndex>(37, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'currentRow'
        QtMocHelpers::PropertyData<int>(38, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<CheckableListModel, qt_meta_tag_ZN18CheckableListModelE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject CheckableListModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractProxyModel::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18CheckableListModelE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18CheckableListModelE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18CheckableListModelE_t>.metaTypes,
    nullptr
} };

void CheckableListModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<CheckableListModel *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->sourceModelChanged(); break;
        case 1: _t->selectionModelChanged(); break;
        case 2: _t->rootIndexChanged(); break;
        case 3: _t->currentRowChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 4: _t->onModelAboutToBeReset(); break;
        case 5: _t->onModelReset(); break;
        case 6: _t->onDataChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 7: _t->onRowsAboutToBeRemoved((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 8: _t->onRowsRemoved((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 9: _t->onRowsAboutToBeInserted((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 10: _t->onRowsInserted((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 11: _t->onSelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 12: _t->onCurrentChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 13: { QVariant _r = _t->getDataValue((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QVariant*>(_a[0]) = std::move(_r); }  break;
        case 14: { bool _r = _t->setDataValue((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QVariant>>(_a[3])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 15: { QModelIndex _r = _t->modelIndex((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 16: { QModelIndex _r = _t->parentModelIndex();
            if (_a[0]) *reinterpret_cast<QModelIndex*>(_a[0]) = std::move(_r); }  break;
        case 17: { bool _r = _t->hasModelChildren((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])));
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (CheckableListModel::*)()>(_a, &CheckableListModel::sourceModelChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (CheckableListModel::*)()>(_a, &CheckableListModel::selectionModelChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (CheckableListModel::*)()>(_a, &CheckableListModel::rootIndexChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (CheckableListModel::*)(int )>(_a, &CheckableListModel::currentRowChanged, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QObject**>(_v) = _t->sourceModel(); break;
        case 1: *reinterpret_cast<QObject**>(_v) = _t->selectionModel(); break;
        case 2: *reinterpret_cast<QModelIndex*>(_v) = _t->rootIndex(); break;
        case 3: *reinterpret_cast<int*>(_v) = _t->currentRow(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setSourceModelObject(*reinterpret_cast<QObject**>(_v)); break;
        case 1: _t->setSelectionModelObject(*reinterpret_cast<QObject**>(_v)); break;
        case 2: _t->setRootIndex(*reinterpret_cast<QModelIndex*>(_v)); break;
        case 3: _t->setCurrentRow(*reinterpret_cast<int*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *CheckableListModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CheckableListModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18CheckableListModelE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractProxyModel::qt_metacast(_clname);
}

int CheckableListModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractProxyModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    return _id;
}

// SIGNAL 0
void CheckableListModel::sourceModelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void CheckableListModel::selectionModelChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void CheckableListModel::rootIndexChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void CheckableListModel::currentRowChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
