/****************************************************************************
** Meta object code from reading C++ file 'tagsearcher.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/core/model/tagsearcher.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tagsearcher.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN11TagSearcherE_t {};
} // unnamed namespace

template <> constexpr inline auto TagSearcher::qt_create_metaobjectdata<qt_meta_tag_ZN11TagSearcherE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TagSearcher",
        "textFound",
        "",
        "textReplaced",
        "progress",
        "msg",
        "abort",
        "find",
        "TagSearcher::Parameters",
        "params",
        "replace",
        "replaceAll",
        "searchNextFile",
        "QPersistentModelIndex",
        "index",
        "replaceThenFindNext"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'textFound'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'textReplaced'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'progress'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Slot 'abort'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'find'
        QtMocHelpers::SlotData<void(const TagSearcher::Parameters &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'replace'
        QtMocHelpers::SlotData<void(const TagSearcher::Parameters &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'replaceAll'
        QtMocHelpers::SlotData<void(const TagSearcher::Parameters &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 9 },
        }}),
        // Slot 'searchNextFile'
        QtMocHelpers::SlotData<void(const QPersistentModelIndex &)>(12, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Slot 'replaceThenFindNext'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TagSearcher, qt_meta_tag_ZN11TagSearcherE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TagSearcher::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11TagSearcherE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11TagSearcherE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11TagSearcherE_t>.metaTypes,
    nullptr
} };

void TagSearcher::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TagSearcher *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->textFound(); break;
        case 1: _t->textReplaced(); break;
        case 2: _t->progress((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->abort(); break;
        case 4: _t->find((*reinterpret_cast<std::add_pointer_t<TagSearcher::Parameters>>(_a[1]))); break;
        case 5: _t->replace((*reinterpret_cast<std::add_pointer_t<TagSearcher::Parameters>>(_a[1]))); break;
        case 6: _t->replaceAll((*reinterpret_cast<std::add_pointer_t<TagSearcher::Parameters>>(_a[1]))); break;
        case 7: _t->searchNextFile((*reinterpret_cast<std::add_pointer_t<QPersistentModelIndex>>(_a[1]))); break;
        case 8: _t->replaceThenFindNext(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TagSearcher::*)()>(_a, &TagSearcher::textFound, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagSearcher::*)()>(_a, &TagSearcher::textReplaced, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TagSearcher::*)(const QString & )>(_a, &TagSearcher::progress, 2))
            return;
    }
}

const QMetaObject *TagSearcher::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TagSearcher::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11TagSearcherE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "IAbortable"))
        return static_cast< IAbortable*>(this);
    return QObject::qt_metacast(_clname);
}

int TagSearcher::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void TagSearcher::textFound()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TagSearcher::textReplaced()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void TagSearcher::progress(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}
QT_WARNING_POP
