/****************************************************************************
** Meta object code from reading C++ file 'frame.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/core/tags/frame.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'frame.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN5FrameE_t {};
} // unnamed namespace

template <> constexpr inline auto Frame::qt_create_metaobjectdata<qt_meta_tag_ZN5FrameE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Frame",
        "Type",
        "FT_Title",
        "FT_FirstFrame",
        "FT_Artist",
        "FT_Album",
        "FT_Comment",
        "FT_Date",
        "FT_Track",
        "FT_Genre",
        "FT_LastV1Frame",
        "FT_AlbumArtist",
        "FT_Arranger",
        "FT_Author",
        "FT_Bpm",
        "FT_CatalogNumber",
        "FT_Compilation",
        "FT_Composer",
        "FT_Conductor",
        "FT_Copyright",
        "FT_Disc",
        "FT_EncodedBy",
        "FT_EncoderSettings",
        "FT_EncodingTime",
        "FT_Grouping",
        "FT_InitialKey",
        "FT_Isrc",
        "FT_Language",
        "FT_Lyricist",
        "FT_Lyrics",
        "FT_Media",
        "FT_Mood",
        "FT_OriginalAlbum",
        "FT_OriginalArtist",
        "FT_OriginalDate",
        "FT_Description",
        "FT_Performer",
        "FT_Picture",
        "FT_Publisher",
        "FT_ReleaseCountry",
        "FT_Remixer",
        "FT_SortAlbum",
        "FT_SortAlbumArtist",
        "FT_SortArtist",
        "FT_SortComposer",
        "FT_SortName",
        "FT_Subtitle",
        "FT_Website",
        "FT_WWWAudioFile",
        "FT_WWWAudioSource",
        "FT_ReleaseDate",
        "FT_Rating",
        "FT_Work",
        "FT_Custom1",
        "FT_Custom2",
        "FT_Custom3",
        "FT_Custom4",
        "FT_Custom5",
        "FT_Custom6",
        "FT_Custom7",
        "FT_Custom8",
        "FT_LastFrame",
        "FT_Other",
        "FT_UnknownFrame",
        "FieldId",
        "ID_NoField",
        "ID_TextEnc",
        "ID_Text",
        "ID_Url",
        "ID_Data",
        "ID_Description",
        "ID_Owner",
        "ID_Email",
        "ID_Rating",
        "ID_Filename",
        "ID_Language",
        "ID_PictureType",
        "ID_ImageFormat",
        "ID_MimeType",
        "ID_Counter",
        "ID_Id",
        "ID_VolumeAdj",
        "ID_NumBits",
        "ID_VolChgRight",
        "ID_VolChgLeft",
        "ID_PeakVolRight",
        "ID_PeakVolLeft",
        "ID_TimestampFormat",
        "ID_ContentType",
        "ID_Price",
        "ID_Date",
        "ID_Seller",
        "ID_TargetType",
        "ID_Default",
        "ID_TrackId",
        "ID_EditionId",
        "ID_ChapterId",
        "ID_AttachmentId",
        "ID_ImageProperties",
        "ID_Subframe",
        "TextEncoding",
        "TE_ISO8859_1",
        "TE_UTF16",
        "TE_UTF16BE",
        "TE_UTF8",
        "PictureType",
        "PT_Other",
        "PT_Png32Icon",
        "PT_OtherIcon",
        "PT_CoverFront",
        "PT_CoverBack",
        "PT_LeafletPage",
        "PT_Media",
        "PT_LeadArtist",
        "PT_Artist",
        "PT_Conductor",
        "PT_Band",
        "PT_Composer",
        "PT_Lyricist",
        "PT_RecordingLocation",
        "PT_DuringRecording",
        "PT_DuringPerformance",
        "PT_Video",
        "PT_Fish",
        "PT_Illustration",
        "PT_ArtistLogo",
        "PT_PublisherLogo",
        "TagNumber",
        "Tag_1",
        "Tag_2",
        "Tag_3",
        "Tag_NumValues",
        "Tag_Id3v1",
        "Tag_Id3v2",
        "Tag_Picture",
        "TagVersion",
        "TagNone",
        "TagV1",
        "TagV2",
        "TagV3",
        "TagV2V1",
        "TagVAll"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Type'
        QtMocHelpers::EnumData<enum Type>(1, 1, QMC::EnumFlags{}).add({
            {    2, Type::FT_Title },
            {    3, Type::FT_FirstFrame },
            {    4, Type::FT_Artist },
            {    5, Type::FT_Album },
            {    6, Type::FT_Comment },
            {    7, Type::FT_Date },
            {    8, Type::FT_Track },
            {    9, Type::FT_Genre },
            {   10, Type::FT_LastV1Frame },
            {   11, Type::FT_AlbumArtist },
            {   12, Type::FT_Arranger },
            {   13, Type::FT_Author },
            {   14, Type::FT_Bpm },
            {   15, Type::FT_CatalogNumber },
            {   16, Type::FT_Compilation },
            {   17, Type::FT_Composer },
            {   18, Type::FT_Conductor },
            {   19, Type::FT_Copyright },
            {   20, Type::FT_Disc },
            {   21, Type::FT_EncodedBy },
            {   22, Type::FT_EncoderSettings },
            {   23, Type::FT_EncodingTime },
            {   24, Type::FT_Grouping },
            {   25, Type::FT_InitialKey },
            {   26, Type::FT_Isrc },
            {   27, Type::FT_Language },
            {   28, Type::FT_Lyricist },
            {   29, Type::FT_Lyrics },
            {   30, Type::FT_Media },
            {   31, Type::FT_Mood },
            {   32, Type::FT_OriginalAlbum },
            {   33, Type::FT_OriginalArtist },
            {   34, Type::FT_OriginalDate },
            {   35, Type::FT_Description },
            {   36, Type::FT_Performer },
            {   37, Type::FT_Picture },
            {   38, Type::FT_Publisher },
            {   39, Type::FT_ReleaseCountry },
            {   40, Type::FT_Remixer },
            {   41, Type::FT_SortAlbum },
            {   42, Type::FT_SortAlbumArtist },
            {   43, Type::FT_SortArtist },
            {   44, Type::FT_SortComposer },
            {   45, Type::FT_SortName },
            {   46, Type::FT_Subtitle },
            {   47, Type::FT_Website },
            {   48, Type::FT_WWWAudioFile },
            {   49, Type::FT_WWWAudioSource },
            {   50, Type::FT_ReleaseDate },
            {   51, Type::FT_Rating },
            {   52, Type::FT_Work },
            {   53, Type::FT_Custom1 },
            {   54, Type::FT_Custom2 },
            {   55, Type::FT_Custom3 },
            {   56, Type::FT_Custom4 },
            {   57, Type::FT_Custom5 },
            {   58, Type::FT_Custom6 },
            {   59, Type::FT_Custom7 },
            {   60, Type::FT_Custom8 },
            {   61, Type::FT_LastFrame },
            {   62, Type::FT_Other },
            {   63, Type::FT_UnknownFrame },
        }),
        // enum 'FieldId'
        QtMocHelpers::EnumData<enum FieldId>(64, 64, QMC::EnumFlags{}).add({
            {   65, FieldId::ID_NoField },
            {   66, FieldId::ID_TextEnc },
            {   67, FieldId::ID_Text },
            {   68, FieldId::ID_Url },
            {   69, FieldId::ID_Data },
            {   70, FieldId::ID_Description },
            {   71, FieldId::ID_Owner },
            {   72, FieldId::ID_Email },
            {   73, FieldId::ID_Rating },
            {   74, FieldId::ID_Filename },
            {   75, FieldId::ID_Language },
            {   76, FieldId::ID_PictureType },
            {   77, FieldId::ID_ImageFormat },
            {   78, FieldId::ID_MimeType },
            {   79, FieldId::ID_Counter },
            {   80, FieldId::ID_Id },
            {   81, FieldId::ID_VolumeAdj },
            {   82, FieldId::ID_NumBits },
            {   83, FieldId::ID_VolChgRight },
            {   84, FieldId::ID_VolChgLeft },
            {   85, FieldId::ID_PeakVolRight },
            {   86, FieldId::ID_PeakVolLeft },
            {   87, FieldId::ID_TimestampFormat },
            {   88, FieldId::ID_ContentType },
            {   89, FieldId::ID_Price },
            {   90, FieldId::ID_Date },
            {   91, FieldId::ID_Seller },
            {   92, FieldId::ID_TargetType },
            {   93, FieldId::ID_Default },
            {   94, FieldId::ID_TrackId },
            {   95, FieldId::ID_EditionId },
            {   96, FieldId::ID_ChapterId },
            {   97, FieldId::ID_AttachmentId },
            {   98, FieldId::ID_ImageProperties },
            {   99, FieldId::ID_Subframe },
        }),
        // enum 'TextEncoding'
        QtMocHelpers::EnumData<enum TextEncoding>(100, 100, QMC::EnumFlags{}).add({
            {  101, TextEncoding::TE_ISO8859_1 },
            {  102, TextEncoding::TE_UTF16 },
            {  103, TextEncoding::TE_UTF16BE },
            {  104, TextEncoding::TE_UTF8 },
        }),
        // enum 'PictureType'
        QtMocHelpers::EnumData<enum PictureType>(105, 105, QMC::EnumFlags{}).add({
            {  106, PictureType::PT_Other },
            {  107, PictureType::PT_Png32Icon },
            {  108, PictureType::PT_OtherIcon },
            {  109, PictureType::PT_CoverFront },
            {  110, PictureType::PT_CoverBack },
            {  111, PictureType::PT_LeafletPage },
            {  112, PictureType::PT_Media },
            {  113, PictureType::PT_LeadArtist },
            {  114, PictureType::PT_Artist },
            {  115, PictureType::PT_Conductor },
            {  116, PictureType::PT_Band },
            {  117, PictureType::PT_Composer },
            {  118, PictureType::PT_Lyricist },
            {  119, PictureType::PT_RecordingLocation },
            {  120, PictureType::PT_DuringRecording },
            {  121, PictureType::PT_DuringPerformance },
            {  122, PictureType::PT_Video },
            {  123, PictureType::PT_Fish },
            {  124, PictureType::PT_Illustration },
            {  125, PictureType::PT_ArtistLogo },
            {  126, PictureType::PT_PublisherLogo },
        }),
        // enum 'TagNumber'
        QtMocHelpers::EnumData<enum TagNumber>(127, 127, QMC::EnumFlags{}).add({
            {  128, TagNumber::Tag_1 },
            {  129, TagNumber::Tag_2 },
            {  130, TagNumber::Tag_3 },
            {  131, TagNumber::Tag_NumValues },
            {  132, TagNumber::Tag_Id3v1 },
            {  133, TagNumber::Tag_Id3v2 },
            {  134, TagNumber::Tag_Picture },
        }),
        // enum 'TagVersion'
        QtMocHelpers::EnumData<enum TagVersion>(135, 135, QMC::EnumFlags{}).add({
            {  136, TagVersion::TagNone },
            {  137, TagVersion::TagV1 },
            {  138, TagVersion::TagV2 },
            {  139, TagVersion::TagV3 },
            {  140, TagVersion::TagV2V1 },
            {  141, TagVersion::TagVAll },
        }),
    };
    return QtMocHelpers::metaObjectData<Frame, qt_meta_tag_ZN5FrameE_t>(QMC::PropertyAccessInStaticMetaCall, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Frame::staticMetaObject = { {
    nullptr,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5FrameE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5FrameE_t>.data,
    nullptr,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5FrameE_t>.metaTypes,
    nullptr
} };

QT_WARNING_POP
