/****************************************************************************
** Meta object code from reading C++ file 'playlistconfig.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/core/config/playlistconfig.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'playlistconfig.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14PlaylistConfigE_t {};
} // unnamed namespace

template <> constexpr inline auto PlaylistConfig::qt_create_metaobjectdata<qt_meta_tag_ZN14PlaylistConfigE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PlaylistConfig",
        "locationChanged",
        "",
        "PlaylistConfig::PlaylistLocation",
        "location",
        "formatChanged",
        "PlaylistConfig::PlaylistFormat",
        "format",
        "fileNameFormatChanged",
        "fileNameFormat",
        "fileNameFormatsChanged",
        "fileNameFormats",
        "sortTagFieldChanged",
        "sortTagField",
        "infoFormatChanged",
        "infoFormat",
        "useFileNameFormatChanged",
        "useFileNameFormat",
        "onlySelectedFilesChanged",
        "onlySelectedFiles",
        "useSortTagFieldChanged",
        "useSortTagField",
        "useFullPathChanged",
        "useFullPath",
        "writeInfoChanged",
        "writeInfo",
        "windowGeometryChanged",
        "windowGeometry",
        "PlaylistFormat",
        "PF_M3U",
        "PF_PLS",
        "PF_XSPF",
        "PlaylistLocation",
        "PL_CurrentDirectory",
        "PL_EveryDirectory",
        "PL_TopLevelDirectory"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'locationChanged'
        QtMocHelpers::SignalData<void(PlaylistConfig::PlaylistLocation)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'formatChanged'
        QtMocHelpers::SignalData<void(PlaylistConfig::PlaylistFormat)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 6, 7 },
        }}),
        // Signal 'fileNameFormatChanged'
        QtMocHelpers::SignalData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'fileNameFormatsChanged'
        QtMocHelpers::SignalData<void(const QStringList &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 11 },
        }}),
        // Signal 'sortTagFieldChanged'
        QtMocHelpers::SignalData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'infoFormatChanged'
        QtMocHelpers::SignalData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Signal 'useFileNameFormatChanged'
        QtMocHelpers::SignalData<void(bool)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 17 },
        }}),
        // Signal 'onlySelectedFilesChanged'
        QtMocHelpers::SignalData<void(bool)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 19 },
        }}),
        // Signal 'useSortTagFieldChanged'
        QtMocHelpers::SignalData<void(bool)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 21 },
        }}),
        // Signal 'useFullPathChanged'
        QtMocHelpers::SignalData<void(bool)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 23 },
        }}),
        // Signal 'writeInfoChanged'
        QtMocHelpers::SignalData<void(bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
        // Signal 'windowGeometryChanged'
        QtMocHelpers::SignalData<void(const QByteArray &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 27 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'location'
        QtMocHelpers::PropertyData<int>(4, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'format'
        QtMocHelpers::PropertyData<int>(7, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable, 1),
        // property 'fileNameFormat'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'fileNameFormats'
        QtMocHelpers::PropertyData<QStringList>(11, QMetaType::QStringList, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'sortTagField'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
        // property 'infoFormat'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'useFileNameFormat'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'onlySelectedFiles'
        QtMocHelpers::PropertyData<bool>(19, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 7),
        // property 'useSortTagField'
        QtMocHelpers::PropertyData<bool>(21, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'useFullPath'
        QtMocHelpers::PropertyData<bool>(23, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 9),
        // property 'writeInfo'
        QtMocHelpers::PropertyData<bool>(25, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 10),
        // property 'windowGeometry'
        QtMocHelpers::PropertyData<QByteArray>(27, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'PlaylistFormat'
        QtMocHelpers::EnumData<enum PlaylistFormat>(28, 28, QMC::EnumFlags{}).add({
            {   29, PlaylistFormat::PF_M3U },
            {   30, PlaylistFormat::PF_PLS },
            {   31, PlaylistFormat::PF_XSPF },
        }),
        // enum 'PlaylistLocation'
        QtMocHelpers::EnumData<enum PlaylistLocation>(32, 32, QMC::EnumFlags{}).add({
            {   33, PlaylistLocation::PL_CurrentDirectory },
            {   34, PlaylistLocation::PL_EveryDirectory },
            {   35, PlaylistLocation::PL_TopLevelDirectory },
        }),
    };
    return QtMocHelpers::metaObjectData<PlaylistConfig, qt_meta_tag_ZN14PlaylistConfigE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PlaylistConfig::staticMetaObject = { {
    QMetaObject::SuperData::link<StoredConfig<PlaylistConfig>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14PlaylistConfigE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14PlaylistConfigE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14PlaylistConfigE_t>.metaTypes,
    nullptr
} };

void PlaylistConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlaylistConfig *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->locationChanged((*reinterpret_cast<std::add_pointer_t<PlaylistConfig::PlaylistLocation>>(_a[1]))); break;
        case 1: _t->formatChanged((*reinterpret_cast<std::add_pointer_t<PlaylistConfig::PlaylistFormat>>(_a[1]))); break;
        case 2: _t->fileNameFormatChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->fileNameFormatsChanged((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 4: _t->sortTagFieldChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->infoFormatChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->useFileNameFormatChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->onlySelectedFilesChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->useSortTagFieldChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 9: _t->useFullPathChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 10: _t->writeInfoChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 11: _t->windowGeometryChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PlaylistConfig::*)(PlaylistConfig::PlaylistLocation )>(_a, &PlaylistConfig::locationChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistConfig::*)(PlaylistConfig::PlaylistFormat )>(_a, &PlaylistConfig::formatChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistConfig::*)(const QString & )>(_a, &PlaylistConfig::fileNameFormatChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistConfig::*)(const QStringList & )>(_a, &PlaylistConfig::fileNameFormatsChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistConfig::*)(const QString & )>(_a, &PlaylistConfig::sortTagFieldChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistConfig::*)(const QString & )>(_a, &PlaylistConfig::infoFormatChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistConfig::*)(bool )>(_a, &PlaylistConfig::useFileNameFormatChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistConfig::*)(bool )>(_a, &PlaylistConfig::onlySelectedFilesChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistConfig::*)(bool )>(_a, &PlaylistConfig::useSortTagFieldChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistConfig::*)(bool )>(_a, &PlaylistConfig::useFullPathChanged, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistConfig::*)(bool )>(_a, &PlaylistConfig::writeInfoChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlaylistConfig::*)(const QByteArray & )>(_a, &PlaylistConfig::windowGeometryChanged, 11))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->location(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->format(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->fileNameFormat(); break;
        case 3: *reinterpret_cast<QStringList*>(_v) = _t->fileNameFormats(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->sortTagField(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->infoFormat(); break;
        case 6: *reinterpret_cast<bool*>(_v) = _t->useFileNameFormat(); break;
        case 7: *reinterpret_cast<bool*>(_v) = _t->onlySelectedFiles(); break;
        case 8: *reinterpret_cast<bool*>(_v) = _t->useSortTagField(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->useFullPath(); break;
        case 10: *reinterpret_cast<bool*>(_v) = _t->writeInfo(); break;
        case 11: *reinterpret_cast<QByteArray*>(_v) = _t->windowGeometry(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setLocationInt(*reinterpret_cast<int*>(_v)); break;
        case 1: _t->setFormatInt(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setFileNameFormat(*reinterpret_cast<QString*>(_v)); break;
        case 3: _t->setFileNameFormats(*reinterpret_cast<QStringList*>(_v)); break;
        case 4: _t->setSortTagField(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setInfoFormat(*reinterpret_cast<QString*>(_v)); break;
        case 6: _t->setUseFileNameFormat(*reinterpret_cast<bool*>(_v)); break;
        case 7: _t->setOnlySelectedFiles(*reinterpret_cast<bool*>(_v)); break;
        case 8: _t->setUseSortTagField(*reinterpret_cast<bool*>(_v)); break;
        case 9: _t->setUseFullPath(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setWriteInfo(*reinterpret_cast<bool*>(_v)); break;
        case 11: _t->setWindowGeometry(*reinterpret_cast<QByteArray*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *PlaylistConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlaylistConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14PlaylistConfigE_t>.strings))
        return static_cast<void*>(this);
    return StoredConfig<PlaylistConfig>::qt_metacast(_clname);
}

int PlaylistConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StoredConfig<PlaylistConfig>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 12;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void PlaylistConfig::locationChanged(PlaylistConfig::PlaylistLocation _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void PlaylistConfig::formatChanged(PlaylistConfig::PlaylistFormat _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void PlaylistConfig::fileNameFormatChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void PlaylistConfig::fileNameFormatsChanged(const QStringList & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void PlaylistConfig::sortTagFieldChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void PlaylistConfig::infoFormatChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void PlaylistConfig::useFileNameFormatChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void PlaylistConfig::onlySelectedFilesChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void PlaylistConfig::useSortTagFieldChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void PlaylistConfig::useFullPathChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void PlaylistConfig::writeInfoChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void PlaylistConfig::windowGeometryChanged(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}
QT_WARNING_POP
