/****************************************************************************
** Meta object code from reading C++ file 'taggedfileselection.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/core/model/taggedfileselection.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'taggedfileselection.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN19TaggedFileSelectionE_t {};
} // unnamed namespace

template <> constexpr inline auto TaggedFileSelection::qt_create_metaobjectdata<qt_meta_tag_ZN19TaggedFileSelectionE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TaggedFileSelection",
        "emptyChanged",
        "",
        "empty",
        "singleFileSelectedChanged",
        "single",
        "singleFileChanged",
        "fileNameModified",
        "tag",
        "TaggedFileSelectionTagContext*",
        "Frame::TagNumber",
        "tagNr",
        "formatString",
        "Frame::TagVersion",
        "tagVersion",
        "fmt",
        "singleFileSelected",
        "fileNameChanged",
        "fileName",
        "filePath",
        "detailInfo",
        "tagFormatV1",
        "tagFormatV2",
        "picture"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'emptyChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'singleFileSelectedChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Signal 'singleFileChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'fileNameModified'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'tag'
        QtMocHelpers::MethodData<TaggedFileSelectionTagContext *(Frame::TagNumber) const>(8, 2, QMC::AccessPublic, 0x80000000 | 9, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Method 'formatString'
        QtMocHelpers::MethodData<QString(Frame::TagVersion, const QString &) const>(12, 2, QMC::AccessPublic, QMetaType::QString, {{
            { 0x80000000 | 13, 14 }, { QMetaType::QString, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'empty'
        QtMocHelpers::PropertyData<bool>(3, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'singleFileSelected'
        QtMocHelpers::PropertyData<bool>(16, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'fileNameChanged'
        QtMocHelpers::PropertyData<bool>(17, QMetaType::Bool, QMC::DefaultPropertyFlags, 2),
        // property 'fileName'
        QtMocHelpers::PropertyData<QString>(18, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable, 2),
        // property 'filePath'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'detailInfo'
        QtMocHelpers::PropertyData<QString>(20, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'tagFormatV1'
        QtMocHelpers::PropertyData<QString>(21, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'tagFormatV2'
        QtMocHelpers::PropertyData<QString>(22, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
        // property 'picture'
        QtMocHelpers::PropertyData<QByteArray>(23, QMetaType::QByteArray, QMC::DefaultPropertyFlags, 2),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TaggedFileSelection, qt_meta_tag_ZN19TaggedFileSelectionE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TaggedFileSelection::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19TaggedFileSelectionE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19TaggedFileSelectionE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN19TaggedFileSelectionE_t>.metaTypes,
    nullptr
} };

void TaggedFileSelection::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TaggedFileSelection *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->emptyChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->singleFileSelectedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->singleFileChanged(); break;
        case 3: _t->fileNameModified(); break;
        case 4: { TaggedFileSelectionTagContext* _r = _t->tag((*reinterpret_cast<std::add_pointer_t<Frame::TagNumber>>(_a[1])));
            if (_a[0]) *reinterpret_cast<TaggedFileSelectionTagContext**>(_a[0]) = std::move(_r); }  break;
        case 5: { QString _r = _t->formatString((*reinterpret_cast<std::add_pointer_t<Frame::TagVersion>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QString*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TaggedFileSelection::*)(bool )>(_a, &TaggedFileSelection::emptyChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaggedFileSelection::*)(bool )>(_a, &TaggedFileSelection::singleFileSelectedChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaggedFileSelection::*)()>(_a, &TaggedFileSelection::singleFileChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaggedFileSelection::*)()>(_a, &TaggedFileSelection::fileNameModified, 3))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->isEmpty(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isSingleFileSelected(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isFilenameChanged(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->getFilename(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->getFilePath(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->getDetailInfo(); break;
        case 6: *reinterpret_cast<QString*>(_v) = _t->getTagFormatV1(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->getTagFormatV2(); break;
        case 8: *reinterpret_cast<QByteArray*>(_v) = _t->getPicture(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 3: _t->setFilename(*reinterpret_cast<QString*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *TaggedFileSelection::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TaggedFileSelection::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN19TaggedFileSelectionE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TaggedFileSelection::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 6;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void TaggedFileSelection::emptyChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void TaggedFileSelection::singleFileSelectedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void TaggedFileSelection::singleFileChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void TaggedFileSelection::fileNameModified()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}
namespace {
struct qt_meta_tag_ZN29TaggedFileSelectionTagContextE_t {};
} // unnamed namespace

template <> constexpr inline auto TaggedFileSelectionTagContext::qt_create_metaobjectdata<qt_meta_tag_ZN29TaggedFileSelectionTagContextE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "TaggedFileSelectionTagContext",
        "hasTagChanged",
        "",
        "hasTag",
        "tagUsedChanged",
        "used",
        "tagFormatChanged",
        "tagUsed",
        "tagFormat"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'hasTagChanged'
        QtMocHelpers::SignalData<void(bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 3 },
        }}),
        // Signal 'tagUsedChanged'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 5 },
        }}),
        // Signal 'tagFormatChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'hasTag'
        QtMocHelpers::PropertyData<bool>(3, QMetaType::Bool, QMC::DefaultPropertyFlags, 0),
        // property 'tagUsed'
        QtMocHelpers::PropertyData<bool>(7, QMetaType::Bool, QMC::DefaultPropertyFlags, 1),
        // property 'tagFormat'
        QtMocHelpers::PropertyData<QString>(8, QMetaType::QString, QMC::DefaultPropertyFlags, 2),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<TaggedFileSelectionTagContext, qt_meta_tag_ZN29TaggedFileSelectionTagContextE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject TaggedFileSelectionTagContext::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29TaggedFileSelectionTagContextE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29TaggedFileSelectionTagContextE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN29TaggedFileSelectionTagContextE_t>.metaTypes,
    nullptr
} };

void TaggedFileSelectionTagContext::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<TaggedFileSelectionTagContext *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->hasTagChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 1: _t->tagUsedChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->tagFormatChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (TaggedFileSelectionTagContext::*)(bool )>(_a, &TaggedFileSelectionTagContext::hasTagChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaggedFileSelectionTagContext::*)(bool )>(_a, &TaggedFileSelectionTagContext::tagUsedChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (TaggedFileSelectionTagContext::*)()>(_a, &TaggedFileSelectionTagContext::tagFormatChanged, 2))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<bool*>(_v) = _t->hasTag(); break;
        case 1: *reinterpret_cast<bool*>(_v) = _t->isTagUsed(); break;
        case 2: *reinterpret_cast<QString*>(_v) = _t->tagFormat(); break;
        default: break;
        }
    }
}

const QMetaObject *TaggedFileSelectionTagContext::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TaggedFileSelectionTagContext::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN29TaggedFileSelectionTagContextE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TaggedFileSelectionTagContext::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void TaggedFileSelectionTagContext::hasTagChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void TaggedFileSelectionTagContext::tagUsedChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void TaggedFileSelectionTagContext::tagFormatChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
