/****************************************************************************
** Meta object code from reading C++ file 'basemainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/gui/forms/basemainwindow.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'basemainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18BaseMainWindowImplE_t {};
} // unnamed namespace

template <> constexpr inline auto BaseMainWindowImpl::qt_create_metaobjectdata<qt_meta_tag_ZN18BaseMainWindowImplE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "BaseMainWindowImpl",
        "frameEdited",
        "",
        "Frame::TagNumber",
        "tagNr",
        "const Frame*",
        "frame",
        "frameSelected",
        "updateWindowCaption",
        "confirmedOpenDirectory",
        "paths",
        "onDirectoryOpened",
        "slotFileOpen",
        "slotFileOpenDirectory",
        "slotFileReload",
        "slotFileSave",
        "slotFileQuit",
        "slotStatusMsg",
        "text",
        "slotClearStatusMsg",
        "slotPlaylistDialog",
        "slotCreatePlaylist",
        "showPlaylistEditDialog",
        "playlistPath",
        "slotImport",
        "slotTagImport",
        "slotBatchImport",
        "slotBrowseCoverArt",
        "slotExport",
        "slotSettingsAutoHideTags",
        "slotSettingsShowHidePicture",
        "find",
        "findReplace",
        "findOnly",
        "slotRenameDirectory",
        "slotNumberTracks",
        "slotFilter",
        "slotPlayAudio",
        "updateCurrentSelection",
        "applySelectionChange",
        "QItemSelection",
        "selected",
        "deselected",
        "updateGuiControls",
        "renameFile",
        "deleteFile",
        "expandFileList",
        "updateAfterFrameModification",
        "TaggedFile*",
        "taggedFile",
        "showPlayToolBar",
        "expandNextDirectory",
        "QPersistentModelIndex",
        "index",
        "filterProgress",
        "type",
        "fileName",
        "passed",
        "total",
        "applyImportedTrackData",
        "onEditFrameDialogFinished",
        "result",
        "onPlaylistEditDialogFinished",
        "toggleExpanded",
        "QModelIndex",
        "deactivateFindReplace",
        "showFoundText",
        "updateReplacedText",
        "showOperationProgress",
        "name",
        "done",
        "bool*",
        "abort",
        "onItemCountChanged",
        "onSelectionCountChanged"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'frameEdited'
        QtMocHelpers::SignalData<void(Frame::TagNumber, const Frame *)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Signal 'frameSelected'
        QtMocHelpers::SignalData<void(Frame::TagNumber, const Frame *)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 5, 6 },
        }}),
        // Slot 'updateWindowCaption'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'confirmedOpenDirectory'
        QtMocHelpers::SlotData<void(const QStringList &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 10 },
        }}),
        // Slot 'onDirectoryOpened'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFileOpen'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFileOpenDirectory'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFileReload'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFileSave'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFileQuit'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotStatusMsg'
        QtMocHelpers::SlotData<void(const QString &)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 18 },
        }}),
        // Slot 'slotClearStatusMsg'
        QtMocHelpers::SlotData<void()>(19, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPlaylistDialog'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotCreatePlaylist'
        QtMocHelpers::SlotData<bool()>(21, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'showPlaylistEditDialog'
        QtMocHelpers::SlotData<void(const QString &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Slot 'slotImport'
        QtMocHelpers::SlotData<void()>(24, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotTagImport'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotBatchImport'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotBrowseCoverArt'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotExport'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSettingsAutoHideTags'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSettingsShowHidePicture'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'find'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'findReplace'
        QtMocHelpers::SlotData<void(bool)>(32, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 33 },
        }}),
        // Slot 'findReplace'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void),
        // Slot 'slotRenameDirectory'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotNumberTracks'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotFilter'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotPlayAudio'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateCurrentSelection'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'applySelectionChange'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(39, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 40, 41 }, { 0x80000000 | 40, 42 },
        }}),
        // Slot 'updateGuiControls'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'renameFile'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'deleteFile'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'expandFileList'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateAfterFrameModification'
        QtMocHelpers::SlotData<void(TaggedFile *, Frame::TagNumber)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 48, 49 }, { 0x80000000 | 3, 4 },
        }}),
        // Slot 'showPlayToolBar'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'expandNextDirectory'
        QtMocHelpers::SlotData<void(const QPersistentModelIndex &)>(51, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 52, 53 },
        }}),
        // Slot 'filterProgress'
        QtMocHelpers::SlotData<void(int, const QString &, int, int)>(54, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 55 }, { QMetaType::QString, 56 }, { QMetaType::Int, 57 }, { QMetaType::Int, 58 },
        }}),
        // Slot 'applyImportedTrackData'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onEditFrameDialogFinished'
        QtMocHelpers::SlotData<void(int)>(60, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Int, 61 },
        }}),
        // Slot 'onPlaylistEditDialogFinished'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'toggleExpanded'
        QtMocHelpers::SlotData<void(const QModelIndex &)>(63, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 64, 53 },
        }}),
        // Slot 'deactivateFindReplace'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showFoundText'
        QtMocHelpers::SlotData<void()>(66, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateReplacedText'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showOperationProgress'
        QtMocHelpers::SlotData<void(const QString &, int, int, bool *)>(68, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 69 }, { QMetaType::Int, 70 }, { QMetaType::Int, 58 }, { 0x80000000 | 71, 72 },
        }}),
        // Slot 'onItemCountChanged'
        QtMocHelpers::SlotData<void()>(73, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'onSelectionCountChanged'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<BaseMainWindowImpl, qt_meta_tag_ZN18BaseMainWindowImplE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject BaseMainWindowImpl::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18BaseMainWindowImplE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18BaseMainWindowImplE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18BaseMainWindowImplE_t>.metaTypes,
    nullptr
} };

void BaseMainWindowImpl::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<BaseMainWindowImpl *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->frameEdited((*reinterpret_cast<std::add_pointer_t<Frame::TagNumber>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<const Frame*>>(_a[2]))); break;
        case 1: _t->frameSelected((*reinterpret_cast<std::add_pointer_t<Frame::TagNumber>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<const Frame*>>(_a[2]))); break;
        case 2: _t->updateWindowCaption(); break;
        case 3: _t->confirmedOpenDirectory((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 4: _t->onDirectoryOpened(); break;
        case 5: _t->slotFileOpen(); break;
        case 6: _t->slotFileOpenDirectory(); break;
        case 7: _t->slotFileReload(); break;
        case 8: _t->slotFileSave(); break;
        case 9: _t->slotFileQuit(); break;
        case 10: _t->slotStatusMsg((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->slotClearStatusMsg(); break;
        case 12: _t->slotPlaylistDialog(); break;
        case 13: { bool _r = _t->slotCreatePlaylist();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->showPlaylistEditDialog((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->slotImport(); break;
        case 16: _t->slotTagImport(); break;
        case 17: _t->slotBatchImport(); break;
        case 18: _t->slotBrowseCoverArt(); break;
        case 19: _t->slotExport(); break;
        case 20: _t->slotSettingsAutoHideTags(); break;
        case 21: _t->slotSettingsShowHidePicture(); break;
        case 22: _t->find(); break;
        case 23: _t->findReplace((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 24: _t->findReplace(); break;
        case 25: _t->slotRenameDirectory(); break;
        case 26: _t->slotNumberTracks(); break;
        case 27: _t->slotFilter(); break;
        case 28: _t->slotPlayAudio(); break;
        case 29: _t->updateCurrentSelection(); break;
        case 30: _t->applySelectionChange((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 31: _t->updateGuiControls(); break;
        case 32: _t->renameFile(); break;
        case 33: _t->deleteFile(); break;
        case 34: _t->expandFileList(); break;
        case 35: _t->updateAfterFrameModification((*reinterpret_cast<std::add_pointer_t<TaggedFile*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<Frame::TagNumber>>(_a[2]))); break;
        case 36: _t->showPlayToolBar(); break;
        case 37: _t->expandNextDirectory((*reinterpret_cast<std::add_pointer_t<QPersistentModelIndex>>(_a[1]))); break;
        case 38: _t->filterProgress((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[4]))); break;
        case 39: _t->applyImportedTrackData(); break;
        case 40: _t->onEditFrameDialogFinished((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 41: _t->onPlaylistEditDialogFinished(); break;
        case 42: _t->toggleExpanded((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1]))); break;
        case 43: _t->deactivateFindReplace(); break;
        case 44: _t->showFoundText(); break;
        case 45: _t->updateReplacedText(); break;
        case 46: _t->showOperationProgress((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool*>>(_a[4]))); break;
        case 47: _t->onItemCountChanged(); break;
        case 48: _t->onSelectionCountChanged(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (BaseMainWindowImpl::*)(Frame::TagNumber , const Frame * )>(_a, &BaseMainWindowImpl::frameEdited, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (BaseMainWindowImpl::*)(Frame::TagNumber , const Frame * )>(_a, &BaseMainWindowImpl::frameSelected, 1))
            return;
    }
}

const QMetaObject *BaseMainWindowImpl::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BaseMainWindowImpl::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18BaseMainWindowImplE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "IFrameEditor"))
        return static_cast< IFrameEditor*>(this);
    return QObject::qt_metacast(_clname);
}

int BaseMainWindowImpl::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 49)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 49;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 49)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 49;
    }
    return _id;
}

// SIGNAL 0
void BaseMainWindowImpl::frameEdited(Frame::TagNumber _t1, const Frame * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void BaseMainWindowImpl::frameSelected(Frame::TagNumber _t1, const Frame * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}
QT_WARNING_POP
