/****************************************************************************
** Meta object code from reading C++ file 'numbertracksconfig.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kid3-3.10.1/src/core/config/numbertracksconfig.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'numbertracksconfig.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN18NumberTracksConfigE_t {};
} // unnamed namespace

template <> constexpr inline auto NumberTracksConfig::qt_create_metaobjectdata<qt_meta_tag_ZN18NumberTracksConfigE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "NumberTracksConfig",
        "numberTracksDestinationChanged",
        "",
        "Frame::TagVersion",
        "numberTracksDestination",
        "numberTracksStartChanged",
        "numberTracksStart",
        "trackNumberingEnabledChanged",
        "trackNumberingEnabled",
        "directoryCounterResetEnabledChanged",
        "directoryCounterResetEnabled",
        "windowGeometryChanged",
        "windowGeometry"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'numberTracksDestinationChanged'
        QtMocHelpers::SignalData<void(Frame::TagVersion)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'numberTracksStartChanged'
        QtMocHelpers::SignalData<void(int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 },
        }}),
        // Signal 'trackNumberingEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 8 },
        }}),
        // Signal 'directoryCounterResetEnabledChanged'
        QtMocHelpers::SignalData<void(bool)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 10 },
        }}),
        // Signal 'windowGeometryChanged'
        QtMocHelpers::SignalData<void(const QByteArray &)>(11, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 12 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'numberTracksDestination'
        QtMocHelpers::PropertyData<int>(4, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable, 0),
        // property 'numberTracksStart'
        QtMocHelpers::PropertyData<int>(6, QMetaType::Int, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'trackNumberingEnabled'
        QtMocHelpers::PropertyData<bool>(8, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 2),
        // property 'directoryCounterResetEnabled'
        QtMocHelpers::PropertyData<bool>(10, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'windowGeometry'
        QtMocHelpers::PropertyData<QByteArray>(12, QMetaType::QByteArray, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 4),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<NumberTracksConfig, qt_meta_tag_ZN18NumberTracksConfigE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject NumberTracksConfig::staticMetaObject = { {
    QMetaObject::SuperData::link<StoredConfig<NumberTracksConfig>::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18NumberTracksConfigE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18NumberTracksConfigE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN18NumberTracksConfigE_t>.metaTypes,
    nullptr
} };

void NumberTracksConfig::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<NumberTracksConfig *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->numberTracksDestinationChanged((*reinterpret_cast<std::add_pointer_t<Frame::TagVersion>>(_a[1]))); break;
        case 1: _t->numberTracksStartChanged((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 2: _t->trackNumberingEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 3: _t->directoryCounterResetEnabledChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 4: _t->windowGeometryChanged((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (NumberTracksConfig::*)(Frame::TagVersion )>(_a, &NumberTracksConfig::numberTracksDestinationChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (NumberTracksConfig::*)(int )>(_a, &NumberTracksConfig::numberTracksStartChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (NumberTracksConfig::*)(bool )>(_a, &NumberTracksConfig::trackNumberingEnabledChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (NumberTracksConfig::*)(bool )>(_a, &NumberTracksConfig::directoryCounterResetEnabledChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (NumberTracksConfig::*)(const QByteArray & )>(_a, &NumberTracksConfig::windowGeometryChanged, 4))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<int*>(_v) = _t->numberTracksDestination(); break;
        case 1: *reinterpret_cast<int*>(_v) = _t->numberTracksStart(); break;
        case 2: *reinterpret_cast<bool*>(_v) = _t->isTrackNumberingEnabled(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isDirectoryCounterResetEnabled(); break;
        case 4: *reinterpret_cast<QByteArray*>(_v) = _t->windowGeometry(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setNumberTracksDstInt(*reinterpret_cast<int*>(_v)); break;
        case 1: _t->setNumberTracksStart(*reinterpret_cast<int*>(_v)); break;
        case 2: _t->setTrackNumberingEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 3: _t->setDirectoryCounterResetEnabled(*reinterpret_cast<bool*>(_v)); break;
        case 4: _t->setWindowGeometry(*reinterpret_cast<QByteArray*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *NumberTracksConfig::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NumberTracksConfig::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN18NumberTracksConfigE_t>.strings))
        return static_cast<void*>(this);
    return StoredConfig<NumberTracksConfig>::qt_metacast(_clname);
}

int NumberTracksConfig::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StoredConfig<NumberTracksConfig>::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 5)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 5)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 5;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void NumberTracksConfig::numberTracksDestinationChanged(Frame::TagVersion _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void NumberTracksConfig::numberTracksStartChanged(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void NumberTracksConfig::trackNumberingEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void NumberTracksConfig::directoryCounterResetEnabledChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void NumberTracksConfig::windowGeometryChanged(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
